@@ -3,12 +3,31 @@ import re
 import threading
 from typing import Optional
 
-from localstack.constants import _TEST_AWS_ACCOUNT_ID
-from localstack.runtime import hooks
+import moto.core
+
+from localstack.constants import DEFAULT_AWS_ACCOUNT_ID
 
 # Thread local storage for keeping current request & account related info
 REQUEST_CTX_TLS = threading.local()
 
+#
+# Access Key IDs
+#
+
+
+def get_aws_access_key_id() -> Optional[str]:
+    """Return the AWS access key ID for current context."""
+    return getattr(REQUEST_CTX_TLS, "access_key_id", None)
+
+
+def set_aws_access_key_id(access_key_id: str):
+    REQUEST_CTX_TLS.access_key_id = access_key_id
+
+
+#
+# Account IDs
+#
+
 
 def get_aws_account_id() -> str:
     """Return the AWS account ID for the current context."""
@@ -16,11 +35,19 @@ def get_aws_account_id() -> str:
 
 
 def get_default_account_id() -> str:
-    return _TEST_AWS_ACCOUNT_ID
+    return DEFAULT_AWS_ACCOUNT_ID
+
+
+def get_moto_default_account_id() -> str:
+    return moto.core.DEFAULT_ACCOUNT_ID
 
 
 account_id_resolver = get_default_account_id
 
+#
+# Utils
+#
+
 
 def get_account_id_from_access_key_id(access_key_id: str) -> str:
     """Return the Account ID associated the Access Key ID."""
@@ -31,24 +58,3 @@ def get_account_id_from_access_key_id(access_key_id: str) -> str:
         return access_key_id
     else:
         return get_default_account_id()
-
-
-def get_ctx_aws_access_key_id() -> Optional[str]:
-    """Return the AWS access key ID for current context."""
-    return getattr(REQUEST_CTX_TLS, "access_key_id", None)
-
-
-def set_ctx_aws_access_key_id(access_key_id: str):
-    REQUEST_CTX_TLS.access_key_id = access_key_id
-
-
-@hooks.on_infra_start()
-def patch_get_account_id():
-    """Patch Moto's account ID resolver with our own."""
-    import moto.core
-
-    moto.core.models.account_id_resolver = get_aws_account_id
-
-    # Note: Moto templates making use of this constant will not get access to
-    # account ID from the current context
-    moto.core.models.ACCOUNT_ID = moto.core.ACCOUNT_ID = get_aws_account_id()
