@@ -214,35 +214,30 @@ class AccessDeniedException(ServiceException):
     code: str = "AccessDeniedException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[ExceptionMessage]
 
 
 class ConflictException(ServiceException):
     code: str = "ConflictException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[ExceptionMessage]
 
 
 class InternalServiceErrorException(ServiceException):
     code: str = "InternalServiceErrorException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[ExceptionMessage]
 
 
 class InvalidNextTokenException(ServiceException):
     code: str = "InvalidNextTokenException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[String]
 
 
 class InvalidParameterException(ServiceException):
     code: str = "InvalidParameterException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: ExceptionMessage
     FieldName: Optional[String]
 
 
@@ -250,28 +245,24 @@ class InvalidPolicyDocument(ServiceException):
     code: str = "InvalidPolicyDocument"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[ExceptionMessage]
 
 
 class InvalidRequestException(ServiceException):
     code: str = "InvalidRequestException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[ExceptionMessage]
 
 
 class InvalidTagException(ServiceException):
     code: str = "InvalidTagException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[ExceptionMessage]
 
 
 class LimitExceededException(ServiceException):
     code: str = "LimitExceededException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[String]
     ResourceType: Optional[String]
 
 
@@ -279,7 +270,6 @@ class ResourceExistsException(ServiceException):
     code: str = "ResourceExistsException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[String]
     ResourceType: Optional[String]
 
 
@@ -287,7 +277,6 @@ class ResourceInUseException(ServiceException):
     code: str = "ResourceInUseException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[String]
     ResourceType: Optional[String]
 
 
@@ -295,7 +284,6 @@ class ResourceNotFoundException(ServiceException):
     code: str = "ResourceNotFoundException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[String]
     ResourceType: Optional[String]
 
 
@@ -303,7 +291,6 @@ class ResourceUnavailableException(ServiceException):
     code: str = "ResourceUnavailableException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[String]
     ResourceType: Optional[String]
 
 
@@ -311,21 +298,18 @@ class ThrottlingException(ServiceException):
     code: str = "ThrottlingException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[ExceptionMessage]
 
 
 class UnknownResourceException(ServiceException):
     code: str = "UnknownResourceException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[ExceptionMessage]
 
 
 class ValidationException(ServiceException):
     code: str = "ValidationException"
     sender_fault: bool = False
     status_code: int = 400
-    Message: Optional[ExceptionMessage]
 
 
 class Tag(TypedDict, total=False):
