@@ -353,6 +353,7 @@ class CreateTopicInput(ServiceRequest):
     Name: topicName
     Attributes: Optional[TopicAttributesMap]
     Tags: Optional[TagList]
+    DataProtectionPolicy: Optional[attributeValue]
 
 
 class CreateTopicResponse(TypedDict, total=False):
@@ -384,6 +385,14 @@ class Endpoint(TypedDict, total=False):
     Attributes: Optional[MapStringToString]
 
 
+class GetDataProtectionPolicyInput(ServiceRequest):
+    ResourceArn: topicARN
+
+
+class GetDataProtectionPolicyResponse(TypedDict, total=False):
+    DataProtectionPolicy: Optional[attributeValue]
+
+
 class GetEndpointAttributesInput(ServiceRequest):
     EndpointArn: String
 
@@ -644,6 +653,11 @@ class PublishResponse(TypedDict, total=False):
     SequenceNumber: Optional[String]
 
 
+class PutDataProtectionPolicyInput(ServiceRequest):
+    ResourceArn: topicARN
+    DataProtectionPolicy: attributeValue
+
+
 class RemovePermissionInput(ServiceRequest):
     TopicArn: topicARN
     Label: label
@@ -790,6 +804,7 @@ class SnsApi:
         name: topicName,
         attributes: TopicAttributesMap = None,
         tags: TagList = None,
+        data_protection_policy: attributeValue = None,
     ) -> CreateTopicResponse:
         raise NotImplementedError
 
@@ -813,6 +828,12 @@ class SnsApi:
     def delete_topic(self, context: RequestContext, topic_arn: topicARN) -> None:
         raise NotImplementedError
 
+    @handler("GetDataProtectionPolicy")
+    def get_data_protection_policy(
+        self, context: RequestContext, resource_arn: topicARN
+    ) -> GetDataProtectionPolicyResponse:
+        raise NotImplementedError
+
     @handler("GetEndpointAttributes")
     def get_endpoint_attributes(
         self, context: RequestContext, endpoint_arn: String
@@ -938,6 +959,15 @@ class SnsApi:
     ) -> PublishBatchResponse:
         raise NotImplementedError
 
+    @handler("PutDataProtectionPolicy")
+    def put_data_protection_policy(
+        self,
+        context: RequestContext,
+        resource_arn: topicARN,
+        data_protection_policy: attributeValue,
+    ) -> None:
+        raise NotImplementedError
+
     @handler("RemovePermission")
     def remove_permission(self, context: RequestContext, topic_arn: topicARN, label: label) -> None:
         raise NotImplementedError
