@@ -3,7 +3,10 @@ import logging
 from moto.awslambda import models as moto_awslambda_models
 
 from localstack import config
+from localstack.services.awslambda.lambda_api import handle_lambda_url_invocation
+from localstack.services.edge import ROUTER
 from localstack.utils.aws import aws_stack
+from localstack.utils.aws.request_context import AWS_REGION_REGEX
 from localstack.utils.patch import patch
 from localstack.utils.platform import is_linux
 from localstack.utils.strings import to_bytes
@@ -19,6 +22,18 @@ def start_lambda(port=None, asynchronous=False):
     from localstack.services.awslambda import lambda_api, lambda_utils
     from localstack.services.infra import start_local_api
 
+    ROUTER.add(
+        "/",
+        host=f"<api_id>.lambda-url.<regex('{AWS_REGION_REGEX}'):region>.<regex('.*'):server>",
+        endpoint=handle_lambda_url_invocation,
+        defaults={"path": ""},
+    )
+    ROUTER.add(
+        "/<path:path>",
+        host=f"<api_id>.lambda-url.<regex('{AWS_REGION_REGEX}'):region>.<regex('.*'):server>",
+        endpoint=handle_lambda_url_invocation,
+    )
+
     # print a warning if we're not running in Docker but using Docker based LAMBDA_EXECUTOR
     if "docker" in lambda_utils.get_executor_mode() and not config.is_in_docker and not is_linux():
         LOG.warning(
