@@ -3,7 +3,7 @@ import re
 
 from botocore.exceptions import ClientError
 
-from localstack.constants import S3_VIRTUAL_HOSTNAME
+from localstack.constants import S3_STATIC_WEBSITE_HOSTNAME, S3_VIRTUAL_HOSTNAME
 from localstack.services.cloudformation.deployment_utils import (
     PLACEHOLDER_RESOURCE_NAME,
     dump_json_params,
@@ -37,7 +37,8 @@ class S3BucketPolicy(GenericBaseModel):
             "create": {
                 "function": "put_bucket_policy",
                 "parameters": rename_params(
-                    dump_json_params(None, "PolicyDocument"), {"PolicyDocument": "Policy"}
+                    dump_json_params(None, "PolicyDocument"),
+                    {"PolicyDocument": "Policy", "Bucket": "Bucket"},
                 ),
             },
             "delete": {"function": "delete_bucket_policy", "parameters": {"Bucket": "Bucket"}},
@@ -163,11 +164,15 @@ class S3Bucket(GenericBaseModel):
             except ClientError as e:
                 if e.response["Error"]["Message"] == "Not Found":
                     bucket_name = props.get("BucketName")
-                    s3_client.create_bucket(
-                        Bucket=bucket_name,
-                        ACL=convert_acl_cf_to_s3(props.get("AccessControl", "PublicRead")),
-                        CreateBucketConfiguration={"LocationConstraint": aws_stack.get_region()},
-                    )
+                    params = {
+                        "Bucket": bucket_name,
+                        "ACL": convert_acl_cf_to_s3(props.get("AccessControl", "PublicRead")),
+                    }
+                    if aws_stack.get_region() != "us-east-1":
+                        params["CreateBucketConfiguration"] = {
+                            "LocationConstraint": aws_stack.get_region()
+                        }
+                    s3_client.create_bucket(**params)
 
         result = {
             "create": [
@@ -210,7 +215,18 @@ class S3Bucket(GenericBaseModel):
         if attribute_name in ["DomainName", "RegionalDomainName"]:
             bucket_name = self._get_bucket_name()
             return "%s.%s" % (bucket_name, S3_VIRTUAL_HOSTNAME)
+
+        if attribute_name == "WebsiteURL":
+            bucket_name = self.props.get("BucketName")
+            return f"https://{bucket_name}.{S3_STATIC_WEBSITE_HOSTNAME}"
+
         return super(S3Bucket, self).get_cfn_attribute(attribute_name)
 
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        bucket_name = self.props.get("BucketName")
+        if attribute == "Arn":
+            return aws_stack.s3_bucket_arn(bucket_name)
+        return bucket_name
+
     def _get_bucket_name(self):
         return self.props.get("BucketName") or self.resource_id
