@@ -1,11 +1,15 @@
 import json
 import logging
+import re
+from copy import deepcopy
 from typing import Any, Dict, List, Optional
 
+from localstack.aws.accounts import get_aws_account_id
 from localstack.aws.api import CommonServiceException, RequestContext, handler
 from localstack.aws.api.cloudformation import (
     CallAs,
     ChangeSetNameOrId,
+    ChangeSetNotFoundException,
     ClientRequestToken,
     CloudformationApi,
     CreateChangeSetInput,
@@ -60,7 +64,7 @@ from localstack.aws.api.cloudformation import (
     ValidateTemplateInput,
     ValidateTemplateOutput,
 )
-from localstack.services.generic_proxy import RegionBackend
+from localstack.services.cloudformation.stores import CloudFormationStore, cloudformation_stores
 from localstack.utils.aws import aws_stack
 from localstack.utils.cloudformation import template_deployer, template_preparer
 from localstack.utils.cloudformation.template_deployer import NoStackUpdates
@@ -69,7 +73,7 @@ from localstack.utils.cloudformation.template_preparer import (
     prepare_template_body,
     template_to_json,
 )
-from localstack.utils.collections import select_attributes
+from localstack.utils.collections import remove_attributes, select_attributes
 from localstack.utils.json import clone, clone_safe
 from localstack.utils.objects import recurse_object
 from localstack.utils.strings import long_uid, short_uid
@@ -77,6 +81,19 @@ from localstack.utils.time import timestamp_millis
 
 LOG = logging.getLogger(__name__)
 
+ARN_CHANGESET_REGEX = re.compile(
+    r"arn:(aws|aws-us-gov|aws-cn):cloudformation:[-a-zA-Z0-9]+:\d{12}:changeSet/[a-zA-Z][-a-zA-Z0-9]*/[-a-zA-Z0-9:/._+]+"
+)
+ARN_STACK_REGEX = re.compile(
+    r"arn:(aws|aws-us-gov|aws-cn):cloudformation:[-a-zA-Z0-9]+:\d{12}:stack/[a-zA-Z][-a-zA-Z0-9]*/[-a-zA-Z0-9:/._+]+"
+)
+
+
+def get_cloudformation_store(account_id: str = None, region: str = None) -> CloudFormationStore:
+    return cloudformation_stores[account_id or get_aws_account_id()][
+        region or aws_stack.get_region()
+    ]
+
 
 class StackSet:
     """A stack set contains multiple stack instances."""
@@ -107,11 +124,12 @@ class StackInstance:
 
 
 class Stack:
-    def __init__(self, metadata=None, template=None):
+    def __init__(self, metadata=None, template=None, template_body=None):
         if template is None:
             template = {}
         self.metadata = metadata or {}
         self.template = template or {}
+        self.template_body = template_body
         self._template_raw = clone_safe(self.template)
         self.template_original = clone_safe(self.template)
         # initialize resources
@@ -211,7 +229,7 @@ class Stack:
     def _set_resource_status_details(self, resource_id: str, physical_res_id: str = None):
         """Helper function to ensure that the status details for the given resource ID are up-to-date."""
         resource = self.resources.get(resource_id)
-        if resource is None:
+        if resource is None or resource.get("Type") == "Parameter":
             # make sure we delete the states for any non-existing/deleted resources
             self._resource_states.pop(resource_id, None)
             return
@@ -226,6 +244,7 @@ class Stack:
         state["StackName"] = state.get("StackName") or self.stack_name
         state["StackId"] = state.get("StackId") or self.stack_id
         state["ResourceType"] = state.get("ResourceType") or self.resources[resource_id].get("Type")
+        state["Timestamp"] = timestamp_millis()
         return state
 
     def resource_status(self, resource_id: str):
@@ -407,7 +426,7 @@ class Stack:
     @property
     def exports_map(self):
         result = {}
-        for export in CloudFormationRegion.get().exports:
+        for export in get_cloudformation_store().exports:
             result[export["Name"]] = export
         return result
 
@@ -484,40 +503,6 @@ class StackChangeSet(Stack):
         return self.stack.stack_parameters(defaults=defaults)
 
 
-class CloudFormationRegion(RegionBackend):
-    def __init__(self):
-        # maps stack ID to stack details
-        self.stacks: Dict[str, Stack] = {}
-        # maps stack set ID to stack set details
-        self.stack_sets: Dict[str, StackSet] = {}
-
-    @property
-    def exports(self):
-        exports = []
-        output_keys = {}
-        for stack_id, stack in self.stacks.items():
-            for output in stack.outputs_list():
-                export_name = output.get("ExportName")
-                if not export_name:
-                    continue
-                if export_name in output_keys:
-                    # TODO: raise exception on stack creation in case of duplicate exports
-                    LOG.warning(
-                        "Found duplicate export name %s in stacks: %s %s",
-                        export_name,
-                        output_keys[export_name],
-                        stack.stack_id,
-                    )
-                entry = {
-                    "ExportingStackId": stack.stack_id,
-                    "Name": export_name,
-                    "Value": output["OutputValue"],
-                }
-                exports.append(entry)
-                output_keys[export_name] = stack.stack_id
-        return exports
-
-
 def clone_stack_params(stack_params):
     try:
         return clone(stack_params)
@@ -527,14 +512,14 @@ def clone_stack_params(stack_params):
 
 
 def find_stack(stack_name: str) -> Optional[Stack]:
-    state = CloudFormationRegion.get()
+    state = get_cloudformation_store()
     return (
         [s for s in state.stacks.values() if stack_name in [s.stack_name, s.stack_id]] or [None]
     )[0]
 
 
 def find_change_set(cs_name: str, stack_name: Optional[str] = None) -> Optional[StackChangeSet]:
-    state = CloudFormationRegion.get()
+    state = get_cloudformation_store()
     stack = find_stack(stack_name)
     stacks = [stack] if stack else state.stacks.values()
     result = [
@@ -576,11 +561,11 @@ class InternalFailure(CommonServiceException):
 class CloudformationProvider(CloudformationApi):
     @handler("CreateStack", expand=False)
     def create_stack(self, context: RequestContext, request: CreateStackInput) -> CreateStackOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
         template_deployer.prepare_template_body(request)  # TODO: avoid mutating request directly
         template = template_preparer.parse_template(request["TemplateBody"])
         stack_name = template["StackName"] = request.get("StackName")
-        stack = Stack(request, template)
+        stack = Stack(request, template)  # TODO: proper body handling like in create_change_set
 
         # find existing stack with same name, and remove it if this stack is in DELETED state
         existing = ([s for s in state.stacks.values() if s.stack_name == stack_name] or [None])[0]
@@ -651,7 +636,7 @@ class CloudformationProvider(CloudformationApi):
     def describe_stacks(
         self, context: RequestContext, stack_name: StackName = None, next_token: NextToken = None
     ) -> DescribeStacksOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
         stack_list = list(state.stacks.values())
         stacks = [
             s.describe_details()
@@ -671,7 +656,7 @@ class CloudformationProvider(CloudformationApi):
         next_token: NextToken = None,
         stack_status_filter: StackStatusFilter = None,
     ) -> ListStacksOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
 
         stacks = [
             s.describe_details()
@@ -704,12 +689,18 @@ class CloudformationProvider(CloudformationApi):
         template_stage: TemplateStage = None,
     ) -> GetTemplateOutput:
 
-        stack = find_stack(stack_name)
+        stack = None
         if change_set_name:
             stack = find_change_set(stack_name=stack_name, cs_name=change_set_name)
+        else:
+            stack = find_stack(stack_name)
         if not stack:
             return stack_not_found_error(stack_name)
-        return GetTemplateOutput(TemplateBody=json.dumps(stack.latest_template_raw()))
+
+        return GetTemplateOutput(
+            TemplateBody=stack.template_body,
+            StagesAvailable=[TemplateStage.Original, TemplateStage.Processed],
+        )
 
     @handler("GetTemplateSummary", expand=False)
     def get_template_summary(
@@ -773,7 +764,7 @@ class CloudformationProvider(CloudformationApi):
     def create_stack_set(
         self, context: RequestContext, request: CreateStackSetInput
     ) -> CreateStackSetOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
         stack_set = StackSet(request)
         stack_set_id = short_uid()
         stack_set.metadata["StackSetId"] = stack_set_id
@@ -785,7 +776,7 @@ class CloudformationProvider(CloudformationApi):
     def describe_stack_set(
         self, context: RequestContext, stack_set_name: StackSetName, call_as: CallAs = None
     ) -> DescribeStackSetOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
         result = [
             sset.metadata
             for sset in state.stack_sets.values()
@@ -800,7 +791,7 @@ class CloudformationProvider(CloudformationApi):
     def update_stack_set(
         self, context: RequestContext, request: UpdateStackSetInput
     ) -> UpdateStackSetOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
         set_name = request.get("StackSetName")
         stack_set = [sset for sset in state.stack_sets.values() if sset.stack_set_name == set_name]
         if not stack_set:
@@ -821,7 +812,7 @@ class CloudformationProvider(CloudformationApi):
     def delete_stack_set(
         self, context: RequestContext, stack_set_name: StackSetName, call_as: CallAs = None
     ) -> DeleteStackSetOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
         stack_set = [
             sset for sset in state.stack_sets.values() if sset.stack_set_name == stack_set_name
         ]
@@ -838,7 +829,7 @@ class CloudformationProvider(CloudformationApi):
     def list_stack_sets(
         self, context: RequestContext, request: ListStackSetsInput
     ) -> ListStackSetsOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
         result = [sset.metadata for sset in state.stack_sets.values()]
         return ListStackSetsOutput(Summaries=result)
 
@@ -868,6 +859,10 @@ class CloudformationProvider(CloudformationApi):
             )  # TODO: check proper message
 
         prepare_template_body(req_params)  # TODO: function has too many unclear responsibilities
+        if not template_body:
+            template_body = req_params[
+                "TemplateBody"
+            ]  # should then have been set by prepare_template_body
         template = template_preparer.parse_template(req_params["TemplateBody"])
         del req_params["TemplateBody"]  # TODO: stop mutating req_params
         template["StackName"] = stack_name
@@ -886,11 +881,15 @@ class CloudformationProvider(CloudformationApi):
                 raise ValidationError(
                     f"Stack {stack_name} already exists"
                 )  # stack should not exist yet (TODO: check proper message)
-            state = CloudFormationRegion.get()
+            state = get_cloudformation_store()
             empty_stack_template = dict(template)
             empty_stack_template["Resources"] = {}
             req_params_copy = clone_stack_params(req_params)
-            stack = Stack(req_params_copy, empty_stack_template)
+            stack = Stack(
+                req_params_copy,
+                empty_stack_template,
+                template_body=template_body,
+            )
             state.stacks[stack.stack_id] = stack
             stack.set_stack_status("REVIEW_IN_PROGRESS")
         elif change_set_type == "IMPORT":
@@ -940,13 +939,31 @@ class CloudformationProvider(CloudformationApi):
         stack_name: StackNameOrId = None,
         next_token: NextToken = None,
     ) -> DescribeChangeSetOutput:
+
+        # only relevant if change_set_name isn't an ARN
+        if not ARN_CHANGESET_REGEX.match(change_set_name):
+            if not stack_name:
+                raise ValidationError(
+                    "StackName must be specified if ChangeSetName is not specified as an ARN."
+                )
+
+            stack = find_stack(stack_name)
+            if not stack:
+                raise ValidationError(f"Stack [{stack_name}] does not exist")
+
         change_set = find_change_set(change_set_name, stack_name=stack_name)
         if not change_set:
-            return not_found_error(
-                f'Unable to find change set "{change_set_name}" for stack "{stack_name}"'
-            )
+            raise ChangeSetNotFoundException(f"ChangeSet [{change_set_name}] does not exist")
 
-        return change_set.metadata
+        attrs = [
+            "ChangeSetType",
+            "StackStatus",
+            "LastUpdatedTime",
+            "DisableRollback",
+            "EnableTerminationProtection",
+        ]
+        result = remove_attributes(deepcopy(change_set.metadata), attrs)
+        return result
 
     @handler("DeleteChangeSet")
     def delete_change_set(
@@ -955,13 +972,25 @@ class CloudformationProvider(CloudformationApi):
         change_set_name: ChangeSetNameOrId,
         stack_name: StackNameOrId = None,
     ) -> DeleteChangeSetOutput:
+
+        # only relevant if change_set_name isn't an ARN
+        if not ARN_CHANGESET_REGEX.match(change_set_name):
+            if not stack_name:
+                raise ValidationError(
+                    "StackName must be specified if ChangeSetName is not specified as an ARN."
+                )
+
+            stack = find_stack(stack_name)
+            if not stack:
+                raise ValidationError(f"Stack [{stack_name}] does not exist")
+
         change_set = find_change_set(change_set_name, stack_name=stack_name)
         if not change_set:
-            return not_found_error(
-                f'Unable to find change set "{change_set_name}" for stack "{stack_name}"'
-            )
+            raise ChangeSetNotFoundException(f"ChangeSet [{change_set_name}] does not exist")
         change_set.stack.change_sets = [
-            cs for cs in change_set.stack.change_sets if cs.change_set_name != change_set_name
+            cs
+            for cs in change_set.stack.change_sets
+            if change_set_name not in (cs.change_set_name, cs.change_set_id)
         ]
         return DeleteChangeSetOutput()
 
@@ -976,9 +1005,7 @@ class CloudformationProvider(CloudformationApi):
     ) -> ExecuteChangeSetOutput:
         change_set = find_change_set(change_set_name, stack_name=stack_name)
         if not change_set:
-            return not_found_error(
-                f'Unable to find change set "{change_set_name}" for stack "{stack_name}"'
-            )
+            raise ChangeSetNotFoundException(f"ChangeSet [{change_set_name}] does not exist")
         if change_set.metadata.get("ExecutionStatus") != ExecutionStatus.AVAILABLE:
             LOG.debug("Change set %s not in execution status 'AVAILABLE'", change_set_name)
             raise InvalidChangeSetStatusException(
@@ -1017,7 +1044,7 @@ class CloudformationProvider(CloudformationApi):
         context: RequestContext,
         request: CreateStackInstancesInput,
     ) -> CreateStackInstancesOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
 
         set_name = request.get("StackSetName")
         stack_set = [sset for sset in state.stack_sets.values() if sset.stack_set_name == set_name]
@@ -1078,7 +1105,7 @@ class CloudformationProvider(CloudformationApi):
         request: ListStackInstancesInput,
     ) -> ListStackInstancesOutput:
         set_name = request.get("StackSetName")
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
         stack_set = [sset for sset in state.stack_sets.values() if sset.stack_set_name == set_name]
         if not stack_set:
             return not_found_error(f'Stack set named "{set_name}" does not exist')
@@ -1091,14 +1118,14 @@ class CloudformationProvider(CloudformationApi):
     def list_exports(
         self, context: RequestContext, next_token: NextToken = None
     ) -> ListExportsOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
         return ListExportsOutput(Exports=state.exports)
 
     @handler("ListImports")
     def list_imports(
         self, context: RequestContext, export_name: ExportName, next_token: NextToken = None
     ) -> ListImportsOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
 
         importing_stack_names = []
         for stack in state.stacks.values():
@@ -1111,7 +1138,7 @@ class CloudformationProvider(CloudformationApi):
     def describe_stack_events(
         self, context: RequestContext, stack_name: StackName = None, next_token: NextToken = None
     ) -> DescribeStackEventsOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
 
         events = []
         for stack_id, stack in state.stacks.items():
@@ -1152,6 +1179,8 @@ class CloudformationProvider(CloudformationApi):
             for res_id, res_status in stack.resource_states.items()
             if logical_resource_id in [res_id, None]
         ]
+        for status in statuses:
+            status.setdefault("DriftInformation", {"StackResourceDriftStatus": "NOT_CHECKED"})
         return DescribeStackResourcesOutput(StackResources=statuses)
 
     @handler("ListStackResources")
@@ -1159,7 +1188,13 @@ class CloudformationProvider(CloudformationApi):
         self, context: RequestContext, stack_name: StackName, next_token: NextToken = None
     ) -> ListStackResourcesOutput:
         result = self.describe_stack_resources(context, stack_name)
-        return ListStackResourcesOutput(StackResourceSummaries=result.pop("StackResources"))
+
+        resources = deepcopy(result.get("StackResources", []))
+        for resource in resources:
+            attrs = ["StackName", "StackId", "Timestamp", "PreviousResourceStatus"]
+            remove_attributes(resource, attrs)
+
+        return ListStackResourcesOutput(StackResourceSummaries=resources)
 
     @handler("DescribeStackSetOperation")
     def describe_stack_set_operation(
@@ -1169,7 +1204,7 @@ class CloudformationProvider(CloudformationApi):
         operation_id: ClientRequestToken,
         call_as: CallAs = None,
     ) -> DescribeStackSetOperationOutput:
-        state = CloudFormationRegion.get()
+        state = get_cloudformation_store()
 
         set_name = stack_set_name
 
