@@ -5,12 +5,15 @@ import random
 import re
 import time
 import traceback
+from binascii import crc32
+from contextlib import contextmanager
 from typing import Dict, List
 
 import requests
 import werkzeug
 
 from localstack import config, constants
+from localstack.aws import handlers
 from localstack.aws.accounts import get_aws_account_id
 from localstack.aws.api import (
     CommonServiceException,
@@ -87,7 +90,6 @@ from localstack.aws.forwarder import HttpFallbackDispatcher, get_request_forward
 from localstack.aws.proxy import AwsApiListener
 from localstack.constants import LOCALHOST
 from localstack.http import Response
-from localstack.services.awslambda import lambda_api
 from localstack.services.dynamodb import server
 from localstack.services.dynamodb.models import DynamoDBStore, dynamodb_stores
 from localstack.services.dynamodb.server import start_dynamodb, wait_for_dynamodb
@@ -95,7 +97,6 @@ from localstack.services.dynamodb.utils import (
     ItemFinder,
     ItemSet,
     SchemaExtractor,
-    calculate_crc32,
     extract_table_name_from_partiql_update,
 )
 from localstack.services.dynamodbstreams import dynamodbstreams_api
@@ -104,7 +105,6 @@ from localstack.services.dynamodbstreams.dynamodbstreams_api import (
 )
 from localstack.services.edge import ROUTER
 from localstack.services.plugins import ServiceLifecycleHook
-from localstack.utils.analytics import event_publisher
 from localstack.utils.aws import aws_stack
 from localstack.utils.collections import select_attributes
 from localstack.utils.common import short_uid, to_bytes
@@ -232,10 +232,12 @@ class SSEUtils:
         if existing_key:
             return existing_key
         kms_client = aws_stack.connect_to_service("kms")
-        key_data = kms_client.create_key(Description="Default key that protects DynamoDB data")
+        key_data = kms_client.create_key(
+            Description="Default key that protects my DynamoDB data when no other key is defined"
+        )
         key_id = key_data["KeyMetadata"]["KeyId"]
 
-        provider.set_key_managed(key_id)
+        provider.set_key_managed(key_id, get_aws_account_id(), aws_stack.get_region())
         MANAGED_KMS_KEYS[aws_stack.get_region()] = key_id
         return key_id
 
@@ -262,44 +264,73 @@ class ValidationException(CommonServiceException):
 
 class DynamoDBApiListener(AwsApiListener):
     def __init__(self, provider=None):
+        # TODO: remove once localstack-ext is refactored
         provider = provider or DynamoDBProvider()
         self.provider = provider
         super().__init__("dynamodb", HttpFallbackDispatcher(provider, provider.get_forward_url))
 
-    def return_response(self, method, path, data, headers, response):
-        if response._content:
-            response_content = to_str(response._content)
-            # fix the table and latest stream ARNs (DynamoDBLocal hardcodes "ddblocal" as the region)
-            content_replaced = re.sub(
-                r'("TableArn"|"LatestStreamArn"|"StreamArn")\s*:\s*"arn:([a-z-]+):dynamodb:ddblocal:([^"]+)"',
-                rf'\1: "arn:\2:dynamodb:{aws_stack.get_region()}:\3"',
-                response_content,
-            )
-            if content_replaced != response_content:
-                response._content = content_replaced
-
-        # set x-amz-crc32 headers required by some client
-        fix_headers_for_updated_response(response)
-
-        # update table definitions
-        data = json.loads(to_str(data))
-        if data and "TableName" in data and "KeySchema" in data:
-            table_definitions = get_store().table_definitions
-            table_definitions[data["TableName"]] = data
-
 
 def get_store(context: RequestContext | None = None) -> DynamoDBStore:
     # todo: create an explicit protocol for to retrieve stores for each provider
     _account_id: str = context.account_id if context else get_aws_account_id()
     _region: str = context.region if context else aws_stack.get_region()
+    # special case: AWS NoSQL Workbench sends "localhost" as region - replace with proper region here
+    if _region == "localhost":
+        _region = aws_stack.get_local_region()
     return dynamodb_stores[_account_id][_region]
 
 
+def find_global_table_region(table_name: str, target_region: str | None = None) -> str | None:
+    """
+    Check if the table is a Version 2019.11.21 table.
+    :param target_region: the region we are looking for
+    :param table_name: the name of the table
+    :return: the original region; `None` if this is not a global table
+    """
+    replicas = get_store().REPLICA_UPDATES.get(table_name)
+    if not replicas:
+        return None
+    original_region = list(replicas.keys())[0]
+    return original_region if target_region in replicas[original_region] else None
+
+
+@contextmanager
+def modify_context_region(context: RequestContext, region: str):
+    """
+    Context manager that modifies the region of a `RequestContext`. At the exit, the context is restored to its
+    original state.
+
+    :param context: the context to modify
+    :param region: the modified region
+    :return: a modified `RequestContext`
+    """
+    original_region = context.region
+    original_authorization = context.request.headers.get("Authorization")
+
+    context.region = region
+    context.request.headers["Authorization"] = re.sub(
+        r"Credential=([^/]+/[^/]+)/(.*?)/",
+        rf"Credential=\1/{region}/",
+        original_authorization or "",
+        flags=re.IGNORECASE,
+    )
+
+    yield context
+
+    # revert the original context
+    context.region = original_region
+    context.request.headers["Authorization"] = original_authorization
+
+
 class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
     def __init__(self):
         self.request_forwarder = get_request_forwarder_http(self.get_forward_url)
 
     def on_after_init(self):
+        # add response processor specific to ddblocal
+        handlers.modify_service_response.append(self.service, self._modify_ddblocal_arns)
+
+        # routes for the shell ui
         ROUTER.add(
             path="/shell",
             endpoint=self.handle_shell_ui_redirect,
@@ -310,12 +341,29 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
             endpoint=self.handle_shell_ui_request,
         )
 
+    def _modify_ddblocal_arns(self, chain, context: RequestContext, response: Response):
+        """A service response handler that modifies the dynamodb backend response."""
+        if response_content := response.get_data(as_text=True):
+            # fix the table and latest stream ARNs (DynamoDBLocal hardcodes "ddblocal" as the region)
+            content_replaced = re.sub(
+                r'("TableArn"|"LatestStreamArn"|"StreamArn")\s*:\s*"arn:([a-z-]+):dynamodb:ddblocal:([^"]+)"',
+                rf'\1: "arn:\2:dynamodb:{aws_stack.get_region()}:\3"',
+                response_content,
+            )
+            if content_replaced != response_content:
+                response.data = content_replaced
+                context.service_response = (
+                    None  # make sure the service response is parsed again later
+                )
+
+        # update x-amz-crc32 header required by some clients
+        response.headers["x-amz-crc32"] = crc32(response.data) & 0xFFFFFFFF
+
     def forward_request(
         self, context: RequestContext, service_request: ServiceRequest = None
     ) -> ServiceResponse:
         # check rate limiting for this request and raise an error, if provisioned throughput is exceeded
         self.check_provisioned_throughput(context.operation.name)
-
         # note: modifying headers in-place here before forwarding the request
         self.prepare_request_headers(context.request.headers)
         return self.request_forwarder(context, service_request)
@@ -353,7 +401,7 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
         # Check if table exists, to avoid error log output from DynamoDBLocal
         table_name = create_table_input["TableName"]
         if self.table_exists(table_name):
-            raise ResourceInUseException("Cannot create preexisting table")
+            raise ResourceInUseException(f"Table already exists: {table_name}")
         billing_mode = create_table_input.get("BillingMode")
         provisioned_throughput = create_table_input.get("ProvisionedThroughput")
         if billing_mode == BillingMode.PAY_PER_REQUEST and provisioned_throughput is not None:
@@ -400,11 +448,6 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
         table_description.pop("Tags", None)
         table_description.pop("BillingMode", None)
 
-        event_publisher.fire_event(
-            event_publisher.EVENT_DYNAMODB_CREATE_TABLE,
-            payload={"n": event_publisher.get_hash(table_name)},
-        )
-
         return result
 
     def delete_table(self, context: RequestContext, table_name: TableName) -> DeleteTableOutput:
@@ -415,10 +458,6 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
         # forward request to backend
         result = self.forward_request(context)
 
-        event_publisher.fire_event(
-            event_publisher.EVENT_DYNAMODB_DELETE_TABLE,
-            payload={"n": event_publisher.get_hash(table_name)},
-        )
         table_arn = result.get("TableDescription", {}).get("TableArn")
         table_arn = self.fix_table_arn(table_arn)
         self.delete_all_event_source_mappings(table_arn)
@@ -427,19 +466,34 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
 
         return result
 
+    def _forward_request(self, context: RequestContext, region: str | None) -> ServiceResponse:
+        if region:
+            with modify_context_region(context, region):
+                return self.forward_request(context)
+        return self.forward_request(context)
+
     def describe_table(self, context: RequestContext, table_name: TableName) -> DescribeTableOutput:
+        global_table_region: str | None = find_global_table_region(
+            table_name=table_name, target_region=context.region
+        )
         # Check if table exists, to avoid error log output from DynamoDBLocal
-        if not self.table_exists(table_name):
+        if not self.table_exists(table_name) and not global_table_region:
             raise ResourceNotFoundException("Cannot do operations on a non-existent table")
 
-        # forward request to backend
-        result = self.forward_request(context)
+        result = self._forward_request(context=context, region=global_table_region)
 
         # update response with additional props
         table_props = get_store(context).table_properties.get(table_name)
         if table_props:
             result.get("Table", {}).update(table_props)
 
+        replicas: Dict = get_store().REPLICA_UPDATES.get(table_name)
+        if replicas and replicas.get(context.region):
+            regions = replicas.get(context.region)
+            result.get("Table", {}).update(
+                {"Replicas": [{"RegionName": region} for region in regions]}
+            )
+
         # update only TableId and SSEDescription if present
         table_definitions = get_store(context).table_definitions.get(table_name)
         if table_definitions:
@@ -477,24 +531,31 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
 
             if update_table_input.get("ReplicaUpdates"):
                 # update local table props (replicas)
-                table_properties = get_store(context).table_properties
-                table_properties[table_name] = table_props = table_properties.get(table_name) or {}
-                table_props["Replicas"] = replicas = table_props.get("Replicas") or []
+                table_properties: Dict = get_store(context).REPLICA_UPDATES
+                table_properties[table_name] = table_replicas = (
+                    table_properties.get(table_name) or {}
+                )
+
+                # get dict of replicas for the table => {original_region: {regions}}
+                original_region = context.region
+                table_replicas[original_region] = region_replicas = (
+                    table_replicas.get(original_region) or set()
+                )
                 for repl_update in update_table_input["ReplicaUpdates"]:
                     for key, details in repl_update.items():
+                        # target region
                         region = details.get("RegionName")
-                        if key == "Create":
-                            details["ReplicaStatus"] = details.get("ReplicaStatus") or "ACTIVE"
-                            replicas.append(details)
-                        if key == "Update":
-                            replica = [r for r in replicas if r.get("RegionName") == region]
-                            if replica:
-                                replica[0].update(details)
-                        if key == "Delete":
-                            table_props["Replicas"] = [
-                                r for r in replicas if r.get("RegionName") != region
-                            ]
-
+                        match key:
+                            case "Create":
+                                table_replicas[original_region].add(region)
+                            case "Update":
+                                table_replicas[original_region] = set(
+                                    [r for r in region_replicas if r == region]
+                                )
+                            case "Delete":
+                                table_replicas[original_region] = set(
+                                    [r for r in region_replicas if r != region]
+                                )
             # update response content
             schema = SchemaExtractor.get_table_schema(table_name)
             return UpdateTableOutput(TableDescription=schema["Table"])
@@ -516,14 +577,16 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
 
     @handler("PutItem", expand=False)
     def put_item(self, context: RequestContext, put_item_input: PutItemInput) -> PutItemOutput:
+        global_table_region: str | None = find_global_table_region(
+            table_name=put_item_input.get("TableName"), target_region=context.region
+        )
         existing_item = None
         table_name = put_item_input["TableName"]
         event_sources_or_streams_enabled = has_event_sources_or_streams_enabled(table_name)
         if event_sources_or_streams_enabled:
             existing_item = ItemFinder.find_existing_item(put_item_input)
 
-        # forward request to backend
-        result = self.forward_request(context, put_item_input)
+        result = self._forward_request(context=context, region=global_table_region)
 
         # Get stream specifications details for the table
         if event_sources_or_streams_enabled:
@@ -560,7 +623,7 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
             existing_item = ItemFinder.find_existing_item(delete_item_input)
 
         # forward request to backend
-        result = self.forward_request(context, delete_item_input)
+        result = self.forward_request(context)
 
         # determine and forward stream record
         if existing_item:
@@ -597,7 +660,7 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
             existing_item = ItemFinder.find_existing_item(update_item_input)
 
         # forward request to backend
-        result = self.forward_request(context, update_item_input)
+        result = self.forward_request(context)
 
         # construct and forward stream record
         if event_sources_or_streams_enabled:
@@ -622,7 +685,10 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
 
     @handler("GetItem", expand=False)
     def get_item(self, context: RequestContext, get_item_input: GetItemInput) -> GetItemOutput:
-        result = self.forward_request(context)
+        global_table_region: str | None = find_global_table_region(
+            table_name=get_item_input.get("TableName"), target_region=context.region
+        )
+        result = self._forward_request(context=context, region=global_table_region)
         self.fix_consumed_capacity(get_item_input, result)
         return result
 
@@ -1016,7 +1082,7 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
         return aws_stack.dynamodb_table_exists(table_name)
 
     @staticmethod
-    def prepare_request_headers(headers):
+    def prepare_request_headers(headers: Dict):
         def _replace(regex, replace):
             headers["Authorization"] = re.sub(
                 regex, replace, headers.get("Authorization") or "", flags=re.IGNORECASE
@@ -1025,6 +1091,7 @@ class DynamoDBProvider(DynamodbApi, ServiceLifecycleHook):
         # Note: We need to ensure that the same access key is used here for all requests,
         # otherwise DynamoDBLocal stores tables/items in separate namespaces
         _replace(r"Credential=[^/]+/", rf"Credential={constants.INTERNAL_AWS_ACCESS_KEY_ID}/")
+
         # Note: The NoSQL Workbench sends "localhost" or "local" as the region name, which we need to fix here
         _replace(
             r"Credential=([^/]+/[^/]+)/local(host)?/",
@@ -1295,7 +1362,10 @@ def has_event_sources_or_streams_enabled(table_name: str, cache: Dict = None):
     cached = cache.get(table_arn)
     if isinstance(cached, bool):
         return cached
-    sources = lambda_api.get_event_sources(source_arn=table_arn)
+    lambda_client = aws_stack.connect_to_service("lambda")
+    sources = lambda_client.list_event_source_mappings(EventSourceArn=table_arn)[
+        "EventSourceMappings"
+    ]
     result = False
     if sources:
         result = True
@@ -1372,11 +1442,6 @@ def get_updated_records(table_name: str, existing_items: List) -> List:
     return result
 
 
-def fix_headers_for_updated_response(response):
-    response.headers["Content-Length"] = len(to_bytes(response.content))
-    response.headers["x-amz-crc32"] = calculate_crc32(response)
-
-
 def create_dynamodb_stream(data, latest_stream_label):
     stream = data["StreamSpecification"]
     enabled = stream.get("StreamEnabled")
