@@ -3,11 +3,11 @@ import os
 from typing import List, Optional
 
 from localstack import config
-from localstack.config import dirs, is_env_true
-from localstack.services import install
-from localstack.services.install import DDB_AGENT_JAR_PATH
+from localstack.config import is_env_true
+from localstack.services.dynamodb.packages import dynamodblocal_package
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import TMP_THREADS, ShellCommandThread, get_free_tcp_port, mkdir
+from localstack.utils.files import rm_rf
 from localstack.utils.run import FuncThread
 from localstack.utils.serving import Server
 from localstack.utils.sync import retry
@@ -45,17 +45,17 @@ class DynamodbServer(Server):
 
     @property
     def jar_path(self) -> str:
-        return f"{dirs.static_libs}/dynamodb/DynamoDBLocal.jar"
+        return f"{dynamodblocal_package.get_installed_dir()}/DynamoDBLocal.jar"
 
     @property
     def library_path(self) -> str:
-        return f"{dirs.static_libs}/dynamodb/DynamoDBLocal_lib"
+        return f"{dynamodblocal_package.get_installed_dir()}/DynamoDBLocal_lib"
 
     def _create_shell_command(self) -> List[str]:
         cmd = [
             "java",
             "-Xmx%s" % self.heap_size,
-            f"-javaagent:{DDB_AGENT_JAR_PATH}",
+            f"-javaagent:{dynamodblocal_package.get_installer().get_ddb_agent_jar_path()}",
             f"-Djava.library.path={self.library_path}",
             "-jar",
             self.jar_path,
@@ -77,7 +77,7 @@ class DynamodbServer(Server):
         return cmd + parameters
 
     def do_start_thread(self) -> FuncThread:
-        install.install_dynamodb_local()
+        dynamodblocal_package.install()
 
         cmd = self._create_shell_command()
         LOG.debug("starting dynamodb process %s", cmd)
@@ -95,20 +95,25 @@ class DynamodbServer(Server):
         LOG.info(line.rstrip())
 
 
-def create_dynamodb_server(port=None) -> DynamodbServer:
+def create_dynamodb_server(
+    port=None, db_path: Optional[str] = None, clean_db_path: bool = False
+) -> DynamodbServer:
     """
     Creates a dynamodb server from the LocalStack configuration.
     """
     port = port or get_free_tcp_port()
-    ddb_data_dir = f"{config.dirs.data}/dynamodb" if config.dirs.data else None
-    return do_create_dynamodb_server(port, ddb_data_dir)
-
-
-def do_create_dynamodb_server(port: int, ddb_data_dir: Optional[str]) -> DynamodbServer:
     server = DynamodbServer(port)
-    if ddb_data_dir:
-        mkdir(ddb_data_dir)
-        absolute_path = os.path.abspath(ddb_data_dir)
+    db_path = f"{config.dirs.data}/dynamodb" if not db_path and config.dirs.data else db_path
+
+    if is_env_true("DYNAMODB_IN_MEMORY"):
+        # the DYNAMODB_IN_MEMORY variable takes precedence and will set the DB path to None which forces inMemory=true
+        db_path = None
+
+    if db_path:
+        if clean_db_path:
+            rm_rf(db_path)
+        mkdir(db_path)
+        absolute_path = os.path.abspath(db_path)
         server.db_path = absolute_path
 
     server.heap_size = config.DYNAMODB_HEAP_SIZE
@@ -116,7 +121,6 @@ def do_create_dynamodb_server(port: int, ddb_data_dir: Optional[str]) -> Dynamod
     server.optimize_db_before_startup = is_env_true("DYNAMODB_OPTIMIZE_DB_BEFORE_STARTUP")
     server.delay_transient_statuses = is_env_true("DYNAMODB_DELAY_TRANSIENT_STATUSES")
     server.cors = os.getenv("DYNAMODB_CORS", None)
-
     return server
 
 
@@ -142,10 +146,10 @@ def check_dynamodb(expect_shutdown=False, print_error=False):
         assert isinstance(out["TableNames"], list)
 
 
-def start_dynamodb(port=None, asynchronous=True, update_listener=None):
+def start_dynamodb(port=None, db_path=None, clean_db_path=False):
     global _server
     if not _server:
-        _server = create_dynamodb_server()
+        _server = create_dynamodb_server(port, db_path, clean_db_path)
 
     _server.start()
 
@@ -154,15 +158,3 @@ def start_dynamodb(port=None, asynchronous=True, update_listener=None):
 
 def get_server():
     return _server
-
-
-def restart_dynamodb():
-    global _server
-    if _server:
-        _server.shutdown()
-        _server.join(timeout=10)
-        _server = None
-
-    LOG.debug("Restarting DynamoDB process ...")
-    start_dynamodb()
-    wait_for_dynamodb()
