@@ -1,12 +1,14 @@
 from datetime import datetime, timezone
 
+from moto.ec2.models import ec2_backends
 from moto.route53resolver.models import Route53ResolverBackend as MotoRoute53ResolverBackend
-from moto.route53resolver.models import route53resolver_backends as moto_route53resolver_backends
+from moto.route53resolver.models import route53resolver_backends
 
 from localstack.aws.api import RequestContext
 from localstack.aws.api.route53resolver import (
     Action,
     AssociateFirewallRuleGroupResponse,
+    AssociateResolverQueryLogConfigResponse,
     BlockOverrideDnsType,
     BlockOverrideDomain,
     BlockOverrideTtl,
@@ -14,58 +16,70 @@ from localstack.aws.api.route53resolver import (
     CreateFirewallDomainListResponse,
     CreateFirewallRuleGroupResponse,
     CreateFirewallRuleResponse,
+    CreateResolverQueryLogConfigResponse,
     CreatorRequestId,
     DeleteFirewallDomainListResponse,
     DeleteFirewallRuleGroupResponse,
     DeleteFirewallRuleResponse,
+    DeleteResolverQueryLogConfigResponse,
+    DestinationArn,
     DisassociateFirewallRuleGroupResponse,
+    DisassociateResolverQueryLogConfigResponse,
+    Filters,
+    FirewallConfig,
     FirewallDomainList,
     FirewallDomainListMetadata,
     FirewallDomainName,
     FirewallDomains,
     FirewallDomainUpdateOperation,
+    FirewallFailOpenStatus,
     FirewallRule,
     FirewallRuleGroup,
     FirewallRuleGroupAssociation,
     FirewallRuleGroupMetadata,
+    GetFirewallConfigResponse,
     GetFirewallDomainListResponse,
     GetFirewallRuleGroupAssociationResponse,
     GetFirewallRuleGroupResponse,
+    GetResolverQueryLogConfigAssociationResponse,
+    GetResolverQueryLogConfigResponse,
     ListDomainMaxResults,
+    ListFirewallConfigsMaxResult,
+    ListFirewallConfigsResponse,
     ListFirewallDomainListsResponse,
     ListFirewallDomainsResponse,
     ListFirewallRuleGroupsResponse,
     ListFirewallRulesResponse,
+    ListResolverQueryLogConfigAssociationsResponse,
+    ListResolverQueryLogConfigsResponse,
     MaxResults,
     MutationProtectionStatus,
     Name,
     NextToken,
     Priority,
+    ResolverQueryLogConfig,
+    ResolverQueryLogConfigAssociation,
+    ResolverQueryLogConfigName,
     ResourceId,
     ResourceNotFoundException,
     Route53ResolverApi,
+    SortByKey,
+    SortOrder,
     TagList,
+    UpdateFirewallConfigResponse,
     UpdateFirewallDomainsResponse,
     UpdateFirewallRuleGroupAssociationResponse,
     UpdateFirewallRuleResponse,
     ValidationException,
 )
-from localstack.services.route53resolver.models import (
-    Route53ResolverBackend,
-    delete_firewall_domain_list,
-    delete_firewall_rule,
-    delete_firewall_rule_group,
-    delete_firewall_rule_group_association,
-    get_firewall_domain,
-    get_firewall_domain_list,
-    get_firewall_rule,
-    get_firewall_rule_group,
-    get_firewall_rule_group_association,
-)
+from localstack.services.route53resolver.models import Route53ResolverBackend
 from localstack.services.route53resolver.utils import (
+    get_resolver_query_log_config_id,
     get_route53_resolver_firewall_domain_list_id,
     get_route53_resolver_firewall_rule_group_association_id,
     get_route53_resolver_firewall_rule_group_id,
+    get_route53_resolver_query_log_config_association_id,
+    validate_destination_arn,
     validate_mutation_protection,
     validate_priority,
 )
@@ -100,21 +114,29 @@ class Route53ResolverProvider(Route53ResolverApi):
             ModificationTime=datetime.now(timezone.utc).isoformat(),
         )
         region_details.firewall_rule_groups[id] = firewall_rule_group
-        moto_route53resolver_backends[context.region].tagger.tag_resource(arn, tags or [])
+        route53resolver_backends[context.account_id][context.region].tagger.tag_resource(
+            arn, tags or []
+        )
         return CreateFirewallRuleGroupResponse(FirewallRuleGroup=firewall_rule_group)
 
     def delete_firewall_rule_group(
         self, context: RequestContext, firewall_rule_group_id: ResourceId
     ) -> DeleteFirewallRuleGroupResponse:
+        region_details = Route53ResolverBackend.get()
         """Delete a Firewall Rule Group."""
-        firewall_rule_group: FirewallRuleGroup = delete_firewall_rule_group(firewall_rule_group_id)
+        firewall_rule_group: FirewallRuleGroup = region_details.delete_firewall_rule_group(
+            firewall_rule_group_id
+        )
         return DeleteFirewallRuleGroupResponse(FirewallRuleGroup=firewall_rule_group)
 
     def get_firewall_rule_group(
         self, context: RequestContext, firewall_rule_group_id: ResourceId
     ) -> GetFirewallRuleGroupResponse:
+        region_details = Route53ResolverBackend.get()
         """Get the details of a Firewall Rule Group."""
-        firewall_rule_group: FirewallRuleGroup = get_firewall_rule_group(firewall_rule_group_id)
+        firewall_rule_group: FirewallRuleGroup = region_details.get_firewall_rule_group(
+            firewall_rule_group_id
+        )
         return GetFirewallRuleGroupResponse(FirewallRuleGroup=firewall_rule_group)
 
     def list_firewall_rule_groups(
@@ -153,14 +175,17 @@ class Route53ResolverProvider(Route53ResolverApi):
             ModificationTime=datetime.now(timezone.utc).isoformat(),
         )
         region_details.firewall_domain_lists[id] = firewall_domain_list
-        moto_route53resolver_backends[context.region].tagger.tag_resource(arn, tags or [])
+        route53resolver_backends[context.account_id][context.region].tagger.tag_resource(
+            arn, tags or []
+        )
         return CreateFirewallDomainListResponse(FirewallDomainList=firewall_domain_list)
 
     def delete_firewall_domain_list(
         self, context: RequestContext, firewall_domain_list_id: ResourceId
     ) -> DeleteFirewallDomainListResponse:
+        region_details = Route53ResolverBackend.get()
         """Delete a Firewall Domain List."""
-        firewall_domain_list: FirewallDomainList = delete_firewall_domain_list(
+        firewall_domain_list: FirewallDomainList = region_details.delete_firewall_domain_list(
             firewall_domain_list_id
         )
         return DeleteFirewallDomainListResponse(FirewallDomainList=firewall_domain_list)
@@ -168,8 +193,11 @@ class Route53ResolverProvider(Route53ResolverApi):
     def get_firewall_domain_list(
         self, context: RequestContext, firewall_domain_list_id: ResourceId
     ) -> GetFirewallDomainListResponse:
+        region_details = Route53ResolverBackend.get()
         """Get the details of a Firewall Domain List."""
-        firewall_domain_list: FirewallDomainList = get_firewall_domain_list(firewall_domain_list_id)
+        firewall_domain_list: FirewallDomainList = region_details.get_firewall_domain_list(
+            firewall_domain_list_id
+        )
         return GetFirewallDomainListResponse(FirewallDomainList=firewall_domain_list)
 
     def list_firewall_domain_lists(
@@ -191,12 +219,17 @@ class Route53ResolverProvider(Route53ResolverApi):
         operation: FirewallDomainUpdateOperation,
         domains: FirewallDomains,
     ) -> UpdateFirewallDomainsResponse:
+        region_details = Route53ResolverBackend.get()
         """Update the domains in a Firewall Domain List."""
         region_details = Route53ResolverBackend.get()
-        firewall_domain_list: FirewallDomainList = get_firewall_domain_list(firewall_domain_list_id)
+        firewall_domain_list: FirewallDomainList = region_details.get_firewall_domain_list(
+            firewall_domain_list_id
+        )
 
-        firewall_domain_list: FirewallDomainList = get_firewall_domain_list(firewall_domain_list_id)
-        firewall_domains = get_firewall_domain(firewall_domain_list_id)
+        firewall_domain_list: FirewallDomainList = region_details.get_firewall_domain_list(
+            firewall_domain_list_id
+        )
+        firewall_domains = region_details.get_firewall_domain(firewall_domain_list_id)
 
         if operation == FirewallDomainUpdateOperation.ADD:
             if not firewall_domains:
@@ -288,8 +321,9 @@ class Route53ResolverProvider(Route53ResolverApi):
         firewall_rule_group_id: ResourceId,
         firewall_domain_list_id: ResourceId,
     ) -> DeleteFirewallRuleResponse:
+        region_details = Route53ResolverBackend.get()
         """Delete a firewall rule"""
-        firewall_rule: FirewallRule = delete_firewall_rule(
+        firewall_rule: FirewallRule = region_details.delete_firewall_rule(
             firewall_rule_group_id, firewall_domain_list_id
         )
         return DeleteFirewallRuleResponse(
@@ -332,8 +366,9 @@ class Route53ResolverProvider(Route53ResolverApi):
         block_override_ttl: BlockOverrideTtl = None,
         name: Name = None,
     ) -> UpdateFirewallRuleResponse:
+        region_details = Route53ResolverBackend.get()
         """Updates a firewall rule"""
-        firewall_rule: FirewallRule = get_firewall_rule(
+        firewall_rule: FirewallRule = region_details.get_firewall_rule(
             firewall_rule_group_id, firewall_domain_list_id
         )
 
@@ -401,7 +436,9 @@ class Route53ResolverProvider(Route53ResolverApi):
             ModificationTime=datetime.now(timezone.utc).isoformat(),
         )
         region_details.firewall_rule_group_associations[id] = firewall_rule_group_association
-        moto_route53resolver_backends[context.region].tagger.tag_resource(arn, tags or [])
+        route53resolver_backends[context.account_id][context.region].tagger.tag_resource(
+            arn, tags or []
+        )
         return AssociateFirewallRuleGroupResponse(
             FirewallRuleGroupAssociation=firewall_rule_group_association
         )
@@ -409,9 +446,12 @@ class Route53ResolverProvider(Route53ResolverApi):
     def disassociate_firewall_rule_group(
         self, context: RequestContext, firewall_rule_group_association_id: ResourceId
     ) -> DisassociateFirewallRuleGroupResponse:
+        region_details = Route53ResolverBackend.get()
         """Disassociate a DNS Firewall rule group from a VPC."""
         firewall_rule_group_association: FirewallRuleGroupAssociation = (
-            delete_firewall_rule_group_association(firewall_rule_group_association_id)
+            region_details.delete_firewall_rule_group_association(
+                firewall_rule_group_association_id
+            )
         )
         return DisassociateFirewallRuleGroupResponse(
             FirewallRuleGroupAssociation=firewall_rule_group_association
@@ -420,9 +460,10 @@ class Route53ResolverProvider(Route53ResolverApi):
     def get_firewall_rule_group_association(
         self, context: RequestContext, firewall_rule_group_association_id: ResourceId
     ) -> GetFirewallRuleGroupAssociationResponse:
+        region_details = Route53ResolverBackend.get()
         """Returns the Firewall Rule Group Association that you specified."""
         firewall_rule_group_association: FirewallRuleGroupAssociation = (
-            get_firewall_rule_group_association(firewall_rule_group_association_id)
+            region_details.get_firewall_rule_group_association(firewall_rule_group_association_id)
         )
         return GetFirewallRuleGroupAssociationResponse(
             FirewallRuleGroupAssociation=firewall_rule_group_association
@@ -436,12 +477,13 @@ class Route53ResolverProvider(Route53ResolverApi):
         mutation_protection: MutationProtectionStatus = None,
         name: Name = None,
     ) -> UpdateFirewallRuleGroupAssociationResponse:
+        region_details = Route53ResolverBackend.get()
         """Updates the specified Firewall Rule Group Association."""
         validate_priority(priority=priority)
         validate_mutation_protection(mutation_protection=mutation_protection)
 
         firewall_rule_group_association: FirewallRuleGroupAssociation = (
-            get_firewall_rule_group_association(firewall_rule_group_association_id)
+            region_details.get_firewall_rule_group_association(firewall_rule_group_association_id)
         )
 
         if priority:
@@ -455,6 +497,205 @@ class Route53ResolverProvider(Route53ResolverApi):
             FirewallRuleGroupAssociation=firewall_rule_group_association
         )
 
+    def create_resolver_query_log_config(
+        self,
+        context: RequestContext,
+        name: ResolverQueryLogConfigName,
+        destination_arn: DestinationArn,
+        creator_request_id: CreatorRequestId,
+        tags: TagList = None,
+    ) -> CreateResolverQueryLogConfigResponse:
+        validate_destination_arn(destination_arn)
+        region_details = Route53ResolverBackend.get()
+        id = get_resolver_query_log_config_id()
+        arn = aws_stack.get_resolver_query_log_config_arn(id)
+        resolver_query_log_config: ResolverQueryLogConfig = ResolverQueryLogConfig(
+            Id=id,
+            Arn=arn,
+            Name=name,
+            AssociationCount=0,
+            Status="CREATED",
+            OwnerId=context.account_id,
+            ShareStatus="NOT_SHARED",
+            DestinationArn=destination_arn,
+            CreatorRequestId=creator_request_id,
+            CreationTime=datetime.now(timezone.utc).isoformat(),
+        )
+        region_details.resolver_query_log_configs[id] = resolver_query_log_config
+        route53resolver_backends[context.account_id][context.region].tagger.tag_resource(
+            arn, tags or []
+        )
+        return CreateResolverQueryLogConfigResponse(
+            ResolverQueryLogConfig=resolver_query_log_config
+        )
+
+    def get_resolver_query_log_config(
+        self, context: RequestContext, resolver_query_log_config_id: ResourceId
+    ) -> GetResolverQueryLogConfigResponse:
+        region_details = Route53ResolverBackend.get()
+        resolver_query_log_config: ResolverQueryLogConfig = (
+            region_details.get_resolver_query_log_config(resolver_query_log_config_id)
+        )
+        return GetResolverQueryLogConfigResponse(ResolverQueryLogConfig=resolver_query_log_config)
+
+    def delete_resolver_query_log_config(
+        self, context: RequestContext, resolver_query_log_config_id: ResourceId
+    ) -> DeleteResolverQueryLogConfigResponse:
+        region_details = Route53ResolverBackend.get()
+        resolver_query_log_config: ResolverQueryLogConfig = (
+            region_details.delete_resolver_query_log_config(resolver_query_log_config_id)
+        )
+        return DeleteResolverQueryLogConfigResponse(
+            ResolverQueryLogConfig=resolver_query_log_config
+        )
+
+    def list_resolver_query_log_configs(
+        self,
+        context: RequestContext,
+        max_results: MaxResults = None,
+        next_token: NextToken = None,
+        filters: Filters = None,
+        sort_by: SortByKey = None,
+        sort_order: SortOrder = None,
+    ) -> ListResolverQueryLogConfigsResponse:
+        region_details = Route53ResolverBackend.get()
+        resolver_query_log_configs = []
+        for resolver_query_log_config in region_details.resolver_query_log_configs.values():
+            resolver_query_log_configs.append(ResolverQueryLogConfig(resolver_query_log_config))
+        return ListResolverQueryLogConfigsResponse(
+            ResolverQueryLogConfigs=resolver_query_log_configs,
+            TotalCount=len(resolver_query_log_configs),
+        )
+
+    def associate_resolver_query_log_config(
+        self,
+        context: RequestContext,
+        resolver_query_log_config_id: ResourceId,
+        resource_id: ResourceId,
+    ) -> AssociateResolverQueryLogConfigResponse:
+
+        region_details = Route53ResolverBackend.get()
+        id = get_route53_resolver_query_log_config_association_id()
+
+        resolver_query_log_config_association: ResolverQueryLogConfigAssociation = (
+            ResolverQueryLogConfigAssociation(
+                Id=id,
+                ResolverQueryLogConfigId=resolver_query_log_config_id,
+                ResourceId=resource_id,
+                Status="ACTIVE",
+                Error="NONE",
+                ErrorMessage="",
+                CreationTime=datetime.now(timezone.utc).isoformat(),
+            )
+        )
+
+        region_details.resolver_query_log_config_associations[
+            id
+        ] = resolver_query_log_config_association
+
+        return AssociateResolverQueryLogConfigResponse(
+            ResolverQueryLogConfigAssociation=resolver_query_log_config_association
+        )
+
+    def disassociate_resolver_query_log_config(
+        self,
+        context: RequestContext,
+        resolver_query_log_config_id: ResourceId,
+        resource_id: ResourceId,
+    ) -> DisassociateResolverQueryLogConfigResponse:
+        region_details = Route53ResolverBackend.get()
+        resolver_query_log_config_association = (
+            region_details.delete_resolver_query_log_config_associations(
+                resolver_query_log_config_id, resource_id
+            )
+        )
+
+        return DisassociateResolverQueryLogConfigResponse(
+            ResolverQueryLogConfigAssociation=resolver_query_log_config_association
+        )
+
+    def get_resolver_query_log_config_association(
+        self, context: RequestContext, resolver_query_log_config_association_id: ResourceId
+    ) -> GetResolverQueryLogConfigAssociationResponse:
+        region_details = Route53ResolverBackend.get()
+        resolver_query_log_config_association: ResolverQueryLogConfigAssociation = (
+            region_details.get_resolver_query_log_config_associations(
+                resolver_query_log_config_association_id
+            )
+        )
+        return GetResolverQueryLogConfigAssociationResponse(
+            ResolverQueryLogConfigAssociation=resolver_query_log_config_association
+        )
+
+    def list_resolver_query_log_config_associations(
+        self,
+        context: RequestContext,
+        max_results: MaxResults = None,
+        next_token: NextToken = None,
+        filters: Filters = None,
+        sort_by: SortByKey = None,
+        sort_order: SortOrder = None,
+    ) -> ListResolverQueryLogConfigAssociationsResponse:
+        region_details = Route53ResolverBackend.get()
+
+        resolver_query_log_config_associations = []
+        for (
+            resolver_query_log_config_association
+        ) in region_details.resolver_query_log_config_associations.values():
+            resolver_query_log_config_associations.append(
+                ResolverQueryLogConfigAssociation(resolver_query_log_config_association)
+            )
+        return ListResolverQueryLogConfigAssociationsResponse(
+            TotalCount=len(resolver_query_log_config_associations),
+            ResolverQueryLogConfigAssociations=resolver_query_log_config_associations,
+        )
+
+    def get_firewall_config(
+        self, context: RequestContext, resource_id: ResourceId
+    ) -> GetFirewallConfigResponse:
+        region_details = Route53ResolverBackend.get()
+        firewall_config = region_details.get_or_create_firewall_config(
+            resource_id, context.region, context.account_id
+        )
+        return GetFirewallConfigResponse(FirewallConfig=firewall_config)
+
+    def list_firewall_configs(
+        self,
+        context: RequestContext,
+        max_results: ListFirewallConfigsMaxResult = None,
+        next_token: NextToken = None,
+    ) -> ListFirewallConfigsResponse:
+        region_details = Route53ResolverBackend.get()
+        firewall_configs = []
+        backend = ec2_backends[context.account_id][context.region]
+        for vpc in backend.vpcs:
+            if vpc not in region_details.firewall_configs:
+                region_details.get_or_create_firewall_config(
+                    vpc, context.region, context.account_id
+                )
+        for firewall_config in region_details.firewall_configs.values():
+            firewall_configs.append(select_from_typed_dict(FirewallConfig, firewall_config))
+        return ListFirewallConfigsResponse(FirewallConfigs=firewall_configs)
+
+    def update_firewall_config(
+        self,
+        context: RequestContext,
+        resource_id: ResourceId,
+        firewall_fail_open: FirewallFailOpenStatus,
+    ) -> UpdateFirewallConfigResponse:
+        region_details = Route53ResolverBackend.get()
+        backend = ec2_backends[context.account_id][context.region]
+        for resource_id in backend.vpcs:
+            if resource_id not in region_details.firewall_configs:
+                firewall_config = region_details.get_or_create_firewall_config(
+                    resource_id, context.region, context.account_id
+                )
+                firewall_config["FirewallFailOpen"] = firewall_fail_open
+            else:
+                firewall_config = region_details.firewall_configs[resource_id]
+                firewall_config["FirewallFailOpen"] = firewall_fail_open
+        return UpdateFirewallConfigResponse(FirewallConfig=firewall_config)
+
 
 @patch(MotoRoute53ResolverBackend._matched_arn)
 def Route53ResolverBackend_matched_arn(fn, self, resource_arn):
@@ -469,4 +710,7 @@ def Route53ResolverBackend_matched_arn(fn, self, resource_arn):
     for firewall_rule_group_association in region_details.firewall_rule_group_associations.values():
         if firewall_rule_group_association.get("Arn") == resource_arn:
             return
+    for resolver_query_log_config in region_details.resolver_query_log_configs.values():
+        if resolver_query_log_config.get("Arn") == resource_arn:
+            return
     fn(self, resource_arn)
