@@ -4,15 +4,15 @@ import json
 import logging
 import re
 import time
-from typing import Dict, Final, Optional, Union
+from typing import Final, Optional, Union
 
 from moto.awslambda.models import LambdaFunction
 from moto.iam.policy_validation import IAMPolicyDocumentValidator
-from moto.secretsmanager import models as secretsmanager_models
-from moto.secretsmanager.models import FakeSecret, SecretsManagerBackend, secretsmanager_backends
+from moto.secretsmanager import utils as secretsmanager_utils
+from moto.secretsmanager.exceptions import SecretNotFoundException as MotoSecretNotFoundException
+from moto.secretsmanager.models import FakeSecret, SecretsManagerBackend
 from moto.secretsmanager.responses import SecretsManagerResponse
 
-from localstack.aws.accounts import get_aws_account_id
 from localstack.aws.api import CommonServiceException, RequestContext, ServiceResponse, handler
 from localstack.aws.api.secretsmanager import (
     CancelRotateSecretRequest,
@@ -49,6 +49,7 @@ from localstack.aws.api.secretsmanager import (
     RotateSecretResponse,
     SecretIdType,
     SecretsmanagerApi,
+    SecretVersionsListEntry,
     StopReplicationToReplicaRequest,
     StopReplicationToReplicaResponse,
     TagResourceRequest,
@@ -88,8 +89,13 @@ class ValidationException(CommonServiceException):
 
 
 class SecretNotFoundException(CommonServiceException):
-    def __init__(self, message: str):
-        super().__init__("SecretNotFoundException", message, 404, True)
+    def __init__(self):
+        super().__init__(
+            "ResourceNotFoundException",
+            "Secrets Manager can't find the specified secret.",
+            400,
+            True,
+        )
 
 
 class SecretsmanagerProvider(SecretsmanagerApi):
@@ -111,6 +117,8 @@ class SecretsmanagerProvider(SecretsmanagerApi):
                 t_secret_id = resource_id[:-7]
             elif resource_id[-1] != "-":
                 t_secret_id += "-"
+            elif resource_id[-1] == "-":
+                t_secret_id = resource_id
         return None if secret_id == t_secret_id else t_secret_id
 
     @staticmethod
@@ -133,7 +141,7 @@ class SecretsmanagerProvider(SecretsmanagerApi):
 
     @staticmethod
     def _call_moto_with_request_secret_id(
-        context: RequestContext, request: Dict
+        context: RequestContext, request: dict
     ) -> ServiceResponse:
         t_secret_id = SecretsmanagerProvider._transform_context_secret_id(
             request.get("SecretId", None)
@@ -318,7 +326,7 @@ class FakeSecretVersionStore(dict):
     def __setitem__(self, key, value):
         self.put_version(key, value, time.time())
 
-    def put_version(self, version_id: str, version: Dict, create_date: Optional[float] = None):
+    def put_version(self, version_id: str, version: dict, create_date: Optional[float] = None):
         if create_date and "createdate" in version:
             version["createdate"] = create_date
         super().__setitem__(version_id, version)
@@ -335,13 +343,15 @@ def fake_secret_set_versions(_, self, versions):
 def moto_smb_get_secret_value(fn, self, secret_id, version_id, version_stage):
     res = fn(self, secret_id, version_id, version_stage)
 
-    secret_id = self.secrets[secret_id]
-    if secret_id:  # Redundant, we know from the response it exists: no exceptions.
-        secret_id.last_accessed_date = today_no_time()
-    else:
-        LOG.warning(
-            f'Expected Secret to exist on non failing GetSecretValue request for SecretId "{secret_id}"'
-        )
+    secret = self.secrets[secret_id]
+
+    # Patch: update last accessed date on get.
+    secret.last_accessed_date = today_no_time()
+
+    # Patch: update version's last accessed date.
+    secret_version = secret.versions.get(version_id or secret.default_version_id)
+    if secret_version:
+        secret_version["last_accessed_date"] = secret.last_accessed_date
 
     return res
 
@@ -358,6 +368,44 @@ def moto_smb_create_secret(fn, self, name, *args, **kwargs):
     return fn(self, name, *args, **kwargs)
 
 
+@patch(SecretsManagerBackend.list_secret_version_ids)
+def moto_smb_list_secret_version_ids(_, self, secret_id, *args, **kwargs):
+    if secret_id not in self.secrets:
+        raise SecretNotFoundException()
+
+    if self.secrets[secret_id].is_deleted():
+        raise InvalidRequestException(
+            "An error occurred (InvalidRequestException) when calling the UpdateSecret operation: "
+            "You can't perform this operation on the secret because it was marked for deletion."
+        )
+
+    secret = self.secrets[secret_id]
+
+    # Patch: output format, report exact createdate instead of current time.
+    versions: list[SecretVersionsListEntry] = list()
+    for version_id, version in secret.versions.items():
+        version_stages = version["version_stages"]
+        entry = SecretVersionsListEntry(
+            CreatedDate=version["createdate"],
+            VersionId=version_id,
+            VersionStages=version_stages,
+        )
+
+        # Patch: bind LastAccessedDate if one exists for this version.
+        last_accessed_date = version.get("last_accessed_date")
+        if last_accessed_date:
+            entry["LastAccessedDate"] = last_accessed_date
+
+        versions.append(entry)
+
+    # Patch: sort versions by date.
+    versions.sort(key=lambda v: v["CreatedDate"], reverse=True)
+
+    response = ListSecretVersionIdsResponse(ARN=secret.arn, Name=secret.name, Versions=versions)
+
+    return json.dumps(response)
+
+
 @patch(FakeSecret.to_dict)
 def fake_secret_to_dict(fn, self):
     res_dict = fn(self)
@@ -369,6 +417,8 @@ def fake_secret_to_dict(fn, self):
         del res_dict["RotationEnabled"]
     if self.auto_rotate_after_days is None and "RotationRules" in res_dict:
         del res_dict["RotationRules"]
+    if not self.tags and "Tags" in res_dict:
+        del res_dict["Tags"]
     for null_field in [key for key, value in res_dict.items() if value is None]:
         del res_dict[null_field]
     return res_dict
@@ -382,15 +432,37 @@ def backend_update_secret(
     description=None,
     **kwargs,
 ):
-    fn(self, secret_id, **kwargs)
+    if secret_id not in self.secrets:
+        raise SecretNotFoundException()
+
+    if self.secrets[secret_id].is_deleted():
+        raise InvalidRequestException(
+            "An error occurred (InvalidRequestException) when calling the UpdateSecret operation: "
+            "You can't perform this operation on the secret because it was marked for deletion."
+        )
+
     secret = self.secrets[secret_id]
+    version_id_t0 = secret.default_version_id
+
+    requires_new_version: bool = any(
+        [kwargs.get("kms_key_id"), kwargs.get("secret_binary"), kwargs.get("secret_string")]
+    )
+    if requires_new_version:
+        fn(self, secret_id, **kwargs)
 
-    # Fix missing update of secret description.
-    # Secret exists if this point is reached.
     if description is not None:
         secret.description = description
 
-    return secret.to_short_dict()
+    version_id_t1 = secret.default_version_id
+
+    resp: UpdateSecretResponse = UpdateSecretResponse()
+    resp["ARN"] = secret.arn
+    resp["Name"] = secret.name
+
+    if version_id_t0 != version_id_t1:
+        resp["VersionId"] = version_id_t1
+
+    return json.dumps(resp)
 
 
 @patch(SecretsManagerResponse.update_secret)
@@ -401,7 +473,7 @@ def response_update_secret(_, self):
     secret_binary = self._get_param("SecretBinary")
     client_request_token = self._get_param("ClientRequestToken")
     kms_key_id = self._get_param("KmsKeyId")
-    return secretsmanager_backends[self.region].update_secret(
+    return self.backend.update_secret(
         secret_id=secret_id,
         description=description,
         secret_string=secret_string,
@@ -418,24 +490,28 @@ def backend_update_secret_version_stage(
     fn(self, secret_id, version_stage, remove_from_version_id, move_to_version_id)
 
     secret = self.secrets[secret_id]
+
+    # Patch: default version is the new AWSCURRENT version
     if version_stage == AWSCURRENT:
         secret.default_version_id = move_to_version_id
 
-        # Ensure only one AWSPREVIOUS tagged version is in the pool.
-        # Remove secret versions with no version stages.
-        versions_no_stages = []
-        update_vid_set = {remove_from_version_id, move_to_version_id}
-        for version_id, version in secret.versions.items():
-            version_stages = version["version_stages"]
-            if version_id not in update_vid_set and AWSPREVIOUS in version_stages:
-                version_stages.remove(AWSPREVIOUS)
-                if not version_stages:
-                    versions_no_stages.append(version_id)
+    versions_no_stages = []
+    update_vid_set = {remove_from_version_id, move_to_version_id}
+    for version_id, version in secret.versions.items():
+        version_stages = version["version_stages"]
 
-        for version_no_stages in versions_no_stages:
-            del secret.versions[version_no_stages]
+        # Patch: ensure only one AWSPREVIOUS tagged version is in the pool.
+        if version_id not in update_vid_set and AWSPREVIOUS in version_stages:
+            version_stages.remove(AWSPREVIOUS)
+            if not version_stages:
+                versions_no_stages.append(version_id)
 
-    return json.dumps({"ARN": secret.arn, "Name": secret.name})
+    # Patch: remove secret versions with no version stages.
+    for version_no_stages in versions_no_stages:
+        del secret.versions[version_no_stages]
+
+    res = UpdateSecretVersionStageResponse(ARN=secret.arn, Name=secret.name)
+    return json.dumps(res)
 
 
 @patch(FakeSecret.reset_default_version)
@@ -472,7 +548,7 @@ def backend_rotate_secret(
     rotation_days = "AutomaticallyAfterDays"
 
     if not self._is_valid_identifier(secret_id):
-        raise SecretNotFoundException(f"Unable to find secret '{secret_id}'")
+        raise SecretNotFoundException()
 
     if self.secrets[secret_id].is_deleted():
         raise InvalidRequestException(
@@ -498,7 +574,7 @@ def backend_rotate_secret(
         get_func_res = lm_client.get_function(FunctionName=rotation_lambda_arn)
         lm_spec = get_func_res["Configuration"]
         lm_spec["Code"] = {"ZipFile": str(short_uid())}
-        rotation_func = LambdaFunction(lm_spec, self.region_name)
+        rotation_func = LambdaFunction(self.account_id, lm_spec, self.region_name)
     except Exception:
         # Fall through to ResourceNotFoundException.
         pass
@@ -576,16 +652,22 @@ def backend_rotate_secret(
     return secret.to_short_dict()
 
 
+@patch(MotoSecretNotFoundException.__init__)
+def moto_secret_not_found_exception_init(fn, self):
+    fn(self)
+    self.code = 400
+
+
 def get_arn_binding_key_for(region: str, secret_id: str) -> str:
     return f"{region}_{secret_id}"
 
 
-def get_arn_binding_for(region, secret_id):
+def get_arn_binding_for(account_id, region, secret_id):
     k = get_arn_binding_key_for(region, secret_id)
     if k not in SECRET_ARN_STORAGE:
         id_string = short_uid()[:6]
         arn = aws_stack.secretsmanager_secret_arn(
-            secret_id, account_id=get_aws_account_id(), region_name=region, random_suffix=id_string
+            secret_id, account_id=account_id, region_name=region, random_suffix=id_string
         )
         SECRET_ARN_STORAGE[k] = arn
     return SECRET_ARN_STORAGE[k]
@@ -614,7 +696,7 @@ def get_resource_policy_model(self, secret_id):
 
 def get_resource_policy_response(self):
     secret_id = self._get_param("SecretId")
-    return secretsmanager_backends[self.region].get_resource_policy(secret_id=secret_id)
+    return self.backend.get_resource_policy(secret_id=secret_id)
 
 
 def delete_resource_policy_model(self, secret_id):
@@ -632,7 +714,7 @@ def delete_resource_policy_model(self, secret_id):
 
 def delete_resource_policy_response(self):
     secret_id = self._get_param("SecretId")
-    return secretsmanager_backends[self.region].delete_resource_policy(secret_id=secret_id)
+    return self.backend.delete_resource_policy(secret_id=secret_id)
 
 
 def put_resource_policy_model(self, secret_id, resource_policy):
@@ -654,13 +736,13 @@ def put_resource_policy_model(self, secret_id, resource_policy):
 def put_resource_policy_response(self):
     secret_id = self._get_param("SecretId")
     resource_policy = self._get_param("ResourcePolicy")
-    return secretsmanager_backends[self.region].put_resource_policy(
+    return self.backend.put_resource_policy(
         secret_id=secret_id, resource_policy=json.loads(resource_policy)
     )
 
 
 def apply_patches():
-    secretsmanager_models.secret_arn = get_arn_binding_for
+    secretsmanager_utils.secret_arn = get_arn_binding_for
     setattr(SecretsManagerBackend, "get_resource_policy", get_resource_policy_model)
     setattr(SecretsManagerResponse, "get_resource_policy", get_resource_policy_response)
 
