@@ -4,7 +4,7 @@ import logging
 import os
 import re
 import time
-from typing import TYPE_CHECKING, Callable, Dict, List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple
 
 import boto3
 import botocore.auth
@@ -20,18 +20,19 @@ from pytest_httpserver import HTTPServer
 
 from localstack import config
 from localstack.aws.accounts import get_aws_account_id
+from localstack.constants import TEST_AWS_ACCESS_KEY_ID, TEST_AWS_SECRET_ACCESS_KEY
 from localstack.testing.aws.cloudformation_utils import load_template_file, render_template
 from localstack.testing.aws.util import get_lambda_logs
 from localstack.utils import testutil
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_stack import create_dynamodb_table
 from localstack.utils.aws.client import SigningHttpClient
-from localstack.utils.common import ensure_list, poll_condition, retry
-from localstack.utils.common import safe_requests as requests
-from localstack.utils.common import short_uid
+from localstack.utils.collections import ensure_list
 from localstack.utils.functions import run_safe
-from localstack.utils.generic.wait_utils import wait_until
+from localstack.utils.http import safe_requests as requests
 from localstack.utils.net import wait_for_port_open
+from localstack.utils.strings import short_uid
+from localstack.utils.sync import ShortCircuitWaitException, poll_condition, retry, wait_until
 from localstack.utils.testutil import start_http_server
 
 if TYPE_CHECKING:
@@ -66,21 +67,44 @@ if TYPE_CHECKING:
     from mypy_boto3_ssm import SSMClient
     from mypy_boto3_stepfunctions import SFNClient
     from mypy_boto3_sts import STSClient
+    from mypy_boto3_transcribe import TranscribeClient
 
 LOG = logging.getLogger(__name__)
 
 
-def _client(service, region_name=None):
-    if os.environ.get("TEST_TARGET") == "AWS_CLOUD":
-        return boto3.client(service, region_name=region_name)
+def is_pro_enabled() -> bool:
+    """Return whether the Pro extensions are enabled, i.e., restricted modules can be imported"""
+    try:
+        import localstack_ext.utils.common  # noqa
+
+        return True
+    except Exception:
+        return False
+
+
+# marker to indicate that a test should be skipped if the Pro extensions are enabled
+skip_if_pro_enabled = pytest.mark.skipif(
+    condition=is_pro_enabled(), reason="skipping, as Pro extensions are enabled"
+)
+
+
+def _client(service, region_name=None, *, additional_config=None):
+    config = botocore.config.Config()
+
     # can't set the timeouts to 0 like in the AWS CLI because the underlying http client requires values > 0
-    config = (
-        botocore.config.Config(
-            connect_timeout=1_000, read_timeout=1_000, retries={"total_max_attempts": 1}
+    if os.environ.get("TEST_DISABLE_RETRIES_AND_TIMEOUTS"):
+        config = config.merge(
+            botocore.config.Config(
+                connect_timeout=1_000, read_timeout=1_000, retries={"total_max_attempts": 1}
+            )
         )
-        if os.environ.get("TEST_DISABLE_RETRIES_AND_TIMEOUTS")
-        else None
-    )
+
+    if additional_config:
+        config = config.merge(additional_config)
+
+    if os.environ.get("TEST_TARGET") == "AWS_CLOUD":
+        return boto3.client(service, region_name=region_name, config=config)
+
     return aws_stack.create_external_boto_client(service, config=config, region_name=region_name)
 
 
@@ -197,6 +221,41 @@ def s3_client() -> "S3Client":
     return _client("s3")
 
 
+@pytest.fixture(scope="class")
+def s3_vhost_client() -> "S3Client":
+    boto_config = botocore.config.Config(s3={"addressing_style": "virtual"})
+    if os.environ.get("TEST_TARGET") == "AWS_CLOUD":
+        return boto3.client("s3", config=boto_config)
+    # can't set the timeouts to 0 like in the AWS CLI because the underlying http client requires values > 0
+    if os.environ.get("TEST_DISABLE_RETRIES_AND_TIMEOUTS"):
+        external_boto_config = botocore.config.Config(
+            connect_timeout=1_000, read_timeout=1_000, retries={"total_max_attempts": 1}
+        )
+        boto_config = boto_config.merge(external_boto_config)
+
+    return aws_stack.create_external_boto_client("s3", config=boto_config)
+
+
+@pytest.fixture(scope="class")
+def s3_presigned_client() -> "S3Client":
+    if os.environ.get("TEST_TARGET") == "AWS_CLOUD":
+        return _client("s3")
+    # can't set the timeouts to 0 like in the AWS CLI because the underlying http client requires values > 0
+    boto_config = (
+        botocore.config.Config(
+            connect_timeout=1_000, read_timeout=1_000, retries={"total_max_attempts": 1}
+        )
+        if os.environ.get("TEST_DISABLE_RETRIES_AND_TIMEOUTS")
+        else None
+    )
+    return aws_stack.connect_to_service(
+        "s3",
+        config=boto_config,
+        aws_access_key_id=TEST_AWS_ACCESS_KEY_ID,
+        aws_secret_access_key=TEST_AWS_SECRET_ACCESS_KEY,
+    )
+
+
 @pytest.fixture(scope="class")
 def s3_resource() -> "S3ServiceResource":
     return _resource("s3")
@@ -322,14 +381,18 @@ def route53_client() -> "Route53Client":
     return _client("route53")
 
 
+@pytest.fixture(scope="class")
+def transcribe_client() -> "TranscribeClient":
+    return _client("transcribe")
+
+
 @pytest.fixture
 def dynamodb_wait_for_table_active(dynamodb_client):
-    def wait_for_table_active(table_name: str):
+    def wait_for_table_active(table_name: str, client=None):
         def wait():
-            return (
-                dynamodb_client.describe_table(TableName=table_name)["Table"]["TableStatus"]
-                == "ACTIVE"
-            )
+            return (client or dynamodb_client).describe_table(TableName=table_name)["Table"][
+                "TableStatus"
+            ] == "ACTIVE"
 
         poll_condition(wait, timeout=30)
 
@@ -342,10 +405,12 @@ def dynamodb_create_table_with_parameters(dynamodb_client, dynamodb_wait_for_tab
 
     def factory(**kwargs):
         if "TableName" not in kwargs:
-            kwargs["TableName"] = "test-table-%s" % short_uid()
+            kwargs["TableName"] = f"test-table-{short_uid()}"
 
         tables.append(kwargs["TableName"])
-        return dynamodb_client.create_table(**kwargs)
+        response = dynamodb_client.create_table(**kwargs)
+        dynamodb_wait_for_table_active(kwargs["TableName"])
+        return response
 
     yield factory
 
@@ -667,6 +732,34 @@ def route53_hosted_zone(route53_client):
             LOG.debug(f"error cleaning up route53 HostedZone {zone}: {e}")
 
 
+@pytest.fixture
+def transcribe_create_job(transcribe_client, s3_client, s3_bucket):
+    def _create_job(audio_file: str, params: Optional[dict[str, Any]] = None) -> str:
+        s3_key = "test-clip.wav"
+
+        if not params:
+            params = {}
+
+        if "TranscriptionJobName" not in params:
+            params["TranscriptionJobName"] = f"test-transcribe-{short_uid()}"
+
+        if "LanguageCode" not in params:
+            params["LanguageCode"] = "en-GB"
+
+        if "Media" not in params:
+            params["Media"] = {"MediaFileUri": f"s3://{s3_bucket}/{s3_key}"}
+
+        # upload test wav to a s3 bucket
+        with open(audio_file, "rb") as f:
+            s3_client.upload_fileobj(f, s3_bucket, s3_key)
+
+        transcribe_client.start_transcription_job(**params)
+
+        return params["TranscriptionJobName"]
+
+    yield _create_job
+
+
 @pytest.fixture
 def kinesis_create_stream(kinesis_client):
     stream_names = []
@@ -702,28 +795,130 @@ def wait_for_stream_ready(kinesis_client):
     return _wait_for_stream_ready
 
 
+@pytest.fixture
+def wait_for_delivery_stream_ready(firehose_client):
+    def _wait_for_stream_ready(delivery_stream_name: str):
+        def is_stream_ready():
+            describe_stream_response = firehose_client.describe_delivery_stream(
+                DeliveryStreamName=delivery_stream_name
+            )
+            return (
+                describe_stream_response["DeliveryStreamDescription"]["DeliveryStreamStatus"]
+                == "ACTIVE"
+            )
+
+        poll_condition(is_stream_ready)
+
+    return _wait_for_stream_ready
+
+
+@pytest.fixture
+def wait_for_dynamodb_stream_ready(dynamodbstreams_client):
+    def _wait_for_stream_ready(stream_arn: str):
+        def is_stream_ready():
+            describe_stream_response = dynamodbstreams_client.describe_stream(StreamArn=stream_arn)
+            return describe_stream_response["StreamDescription"]["StreamStatus"] == "ENABLED"
+
+        poll_condition(is_stream_ready)
+
+    return _wait_for_stream_ready
+
+
 @pytest.fixture()
-def kms_create_key(kms_client):
+def kms_create_key(create_boto_client):
     key_ids = []
 
-    def _create_key(**kwargs):
+    def _create_key(region=None, **kwargs):
         if "Description" not in kwargs:
             kwargs["Description"] = f"test description - {short_uid()}"
-        if "KeyUsage" not in kwargs:
-            kwargs["KeyUsage"] = "ENCRYPT_DECRYPT"
-        key_metadata = kms_client.create_key(**kwargs)["KeyMetadata"]
-        key_ids.append(key_metadata["KeyId"])
+        key_metadata = create_boto_client("kms", region).create_key(**kwargs)["KeyMetadata"]
+        key_ids.append((region, key_metadata["KeyId"]))
         return key_metadata
 
     yield _create_key
 
-    for key_id in key_ids:
+    for region, key_id in key_ids:
         try:
-            kms_client.schedule_key_deletion(KeyId=key_id)
+            create_boto_client("kms", region).schedule_key_deletion(KeyId=key_id)
         except Exception as e:
             LOG.debug("error cleaning up KMS key %s: %s", key_id, e)
 
 
+@pytest.fixture()
+def kms_replicate_key(create_boto_client):
+    key_ids = []
+
+    def _replicate_key(region_from=None, **kwargs):
+        region_to = kwargs.get("ReplicaRegion")
+        key_ids.append((region_to, kwargs.get("KeyId")))
+        return create_boto_client("kms", region_from).replicate_key(**kwargs)
+
+    yield _replicate_key
+
+    for region_to, key_id in key_ids:
+        try:
+            create_boto_client("kms", region_to).schedule_key_deletion(KeyId=key_id)
+        except Exception as e:
+            LOG.debug("error cleaning up KMS key %s: %s", key_id, e)
+
+
+# kms_create_key fixture is used here not just to be able to create aliases without a key specified,
+# but also to make sure that kms_create_key gets executed before and teared down after kms_create_alias -
+# to make sure that we clean up aliases before keys get cleaned up.
+@pytest.fixture()
+def kms_create_alias(kms_client, kms_create_key):
+    aliases = []
+
+    def _create_alias(**kwargs):
+        if "AliasName" not in kwargs:
+            kwargs["AliasName"] = f"alias/{short_uid()}"
+        if "TargetKeyId" not in kwargs:
+            kwargs["TargetKeyId"] = kms_create_key()["KeyId"]
+
+        kms_client.create_alias(**kwargs)
+        aliases.append(kwargs["AliasName"])
+        return kwargs["AliasName"]
+
+    yield _create_alias
+
+    for alias in aliases:
+        try:
+            kms_client.delete_alias(AliasName=alias)
+        except Exception as e:
+            LOG.debug("error cleaning up KMS alias %s: %s", alias, e)
+
+
+@pytest.fixture()
+def kms_create_grant(kms_client, kms_create_key):
+    grants = []
+
+    def _create_grant(**kwargs):
+        # Just a random ARN, since KMS in LocalStack currently doesn't validate GranteePrincipal,
+        # but some GranteePrincipal is required to create a grant.
+        GRANTEE_PRINCIPAL_ARN = (
+            "arn:aws:kms:eu-central-1:123456789876:key/198a5a78-52c3-489f-ac70-b06a4d11027a"
+        )
+
+        if "Operations" not in kwargs:
+            kwargs["Operations"] = ["Decrypt", "Encrypt"]
+        if "GranteePrincipal" not in kwargs:
+            kwargs["GranteePrincipal"] = GRANTEE_PRINCIPAL_ARN
+        if "KeyId" not in kwargs:
+            kwargs["KeyId"] = kms_create_key()["KeyId"]
+
+        grant_id = kms_client.create_grant(**kwargs)["GrantId"]
+        grants.append((grant_id, kwargs["KeyId"]))
+        return grant_id, kwargs["KeyId"]
+
+    yield _create_grant
+
+    for grant_id, key_id in grants:
+        try:
+            kms_client.retire_grant(GrantId=grant_id, KeyId=key_id)
+        except Exception as e:
+            LOG.debug("error cleaning up KMS grant %s: %s", grant_id, e)
+
+
 @pytest.fixture
 def kms_key(kms_create_key):
     return kms_create_key()
@@ -877,6 +1072,7 @@ def deploy_cfn_template(
         parameters: Optional[Dict[str, str]] = None,
         max_wait: Optional[int] = None,
     ) -> DeployResult:
+
         if is_update:
             assert stack_name
         stack_name = stack_name or f"stack-{short_uid()}"
@@ -905,7 +1101,6 @@ def deploy_cfn_template(
 
         assert wait_until(is_change_set_created_and_available(change_set_id), _max_wait=60)
         cfn_client.execute_change_set(ChangeSetName=change_set_id)
-        # TODO: potentially poll for ExecutionStatus=ROLLBACK_COMPLETE here as well, to catch errors early on
         assert wait_until(is_change_set_finished(change_set_id), _max_wait=max_wait or 60)
 
         outputs = cfn_client.describe_stacks(StackName=stack_id)["Stacks"][0].get("Outputs", [])
@@ -923,7 +1118,7 @@ def deploy_cfn_template(
                     == "DELETE_COMPLETE"
                 )
 
-            assert wait_until(_await_stack_delete, _max_wait=60)
+            assert wait_until(_await_stack_delete, _max_wait=max_wait or 60)
             # TODO: fix in localstack. stack should only be in DELETE_COMPLETE state after all resources have been deleted
             time.sleep(2)
 
@@ -1006,9 +1201,18 @@ def _has_stack_status(cfn_client, statuses: List[str]):
 
 @pytest.fixture
 def is_change_set_finished(cfn_client):
-    def _is_change_set_finished(change_set_id: str):
+    def _is_change_set_finished(change_set_id: str, stack_name: Optional[str] = None):
         def _inner():
-            check_set = cfn_client.describe_change_set(ChangeSetName=change_set_id)
+            kwargs = {"ChangeSetName": change_set_id}
+            if stack_name:
+                kwargs["StackName"] = stack_name
+
+            check_set = cfn_client.describe_change_set(**kwargs)
+
+            if check_set.get("ExecutionStatus") == "ROLLBACK_COMPLETE":
+                LOG.warning("Change set failed")
+                raise ShortCircuitWaitException()
+
             return check_set.get("ExecutionStatus") == "EXECUTE_COMPLETE"
 
         return _inner
@@ -1018,14 +1222,16 @@ def is_change_set_finished(cfn_client):
 
 @pytest.fixture
 def wait_until_lambda_ready(lambda_client):
-    def _wait_until_ready(function_name: str, qualifier: str = None):
+    def _wait_until_ready(function_name: str, qualifier: str = None, client=None):
+        client = client or lambda_client
+
         def _is_not_pending():
             kwargs = {}
             if qualifier:
                 kwargs["Qualifier"] = qualifier
             try:
                 result = (
-                    lambda_client.get_function(FunctionName=function_name)["Configuration"]["State"]
+                    client.get_function(FunctionName=function_name)["Configuration"]["State"]
                     != "Pending"
                 )
                 LOG.debug(f"lambda state result: {result=}")
@@ -1075,36 +1281,30 @@ role_policy = """
 
 
 @pytest.fixture
-def create_lambda_function(lambda_client, logs_client, iam_client, wait_until_lambda_ready):
+def create_lambda_function_aws(
+    lambda_client,
+):
     lambda_arns = []
-    role_names_policy_arns = []
-    log_groups = []
-
-    def _create_lambda_function(*args, **kwargs):
-        kwargs["client"] = lambda_client
-        func_name = kwargs.get("func_name")
-        assert func_name
-        del kwargs["func_name"]
-
-        if not kwargs.get("role"):
-            role_name = f"lambda-autogenerated-{short_uid()}"
-            role = iam_client.create_role(
-                RoleName=role_name, AssumeRolePolicyDocument=role_assume_policy
-            )["Role"]
-            policy_name = f"lambda-autogenerated-{short_uid()}"
-            policy_arn = iam_client.create_policy(
-                PolicyName=policy_name, PolicyDocument=role_policy
-            )["Policy"]["Arn"]
-            role_names_policy_arns.append((role_name, policy_arn))
-            iam_client.attach_role_policy(RoleName=role_name, PolicyArn=policy_arn)
-            kwargs["role"] = role["Arn"]
 
+    def _create_lambda_function(**kwargs):
         def _create_function():
-            resp = testutil.create_lambda_function(func_name, **kwargs)
-            lambda_arns.append(resp["CreateFunctionResponse"]["FunctionArn"])
-            wait_until_lambda_ready(function_name=func_name)
-            log_group_name = f"/aws/lambda/{func_name}"
-            log_groups.append(log_group_name)
+            resp = lambda_client.create_function(**kwargs)
+            lambda_arns.append(resp["FunctionArn"])
+
+            def _is_not_pending():
+                try:
+                    result = (
+                        lambda_client.get_function(FunctionName=resp["FunctionName"])[
+                            "Configuration"
+                        ]["State"]
+                        != "Pending"
+                    )
+                    return result
+                except Exception as e:
+                    LOG.error(e)
+                    raise
+
+            wait_until(_is_not_pending)
             return resp
 
         # @AWS, takes about 10s until the role/policy is "active", until then it will fail
@@ -1119,16 +1319,43 @@ def create_lambda_function(lambda_client, logs_client, iam_client, wait_until_la
         except Exception:
             LOG.debug(f"Unable to delete function {arn=} in cleanup")
 
-    for role_name, policy_arn in role_names_policy_arns:
+
+@pytest.fixture
+def create_lambda_function(
+    lambda_client, logs_client, iam_client, wait_until_lambda_ready, lambda_su_role
+):
+    lambda_arns_and_clients = []
+    log_groups = []
+
+    def _create_lambda_function(*args, **kwargs):
+        client = kwargs.get("client") or lambda_client
+        kwargs["client"] = client
+        func_name = kwargs.get("func_name")
+        assert func_name
+        del kwargs["func_name"]
+
+        if not kwargs.get("role"):
+            kwargs["role"] = lambda_su_role
+
+        def _create_function():
+            resp = testutil.create_lambda_function(func_name, **kwargs)
+            lambda_arns_and_clients.append((resp["CreateFunctionResponse"]["FunctionArn"], client))
+            wait_until_lambda_ready(function_name=func_name, client=client)
+            log_group_name = f"/aws/lambda/{func_name}"
+            log_groups.append(log_group_name)
+            return resp
+
+        # @AWS, takes about 10s until the role/policy is "active", until then it will fail
+        # localstack should normally not require the retries and will just continue here
+        return retry(_create_function, retries=3, sleep=4)
+
+    yield _create_lambda_function
+
+    for arn, client in lambda_arns_and_clients:
         try:
-            iam_client.detach_role_policy(RoleName=role_name, PolicyArn=policy_arn)
-            iam_client.delete_role(RoleName=role_name)
+            client.delete_function(FunctionName=arn)
         except Exception:
-            LOG.debug(f"Unable to delete role {role_name=} in cleanup")
-        try:
-            iam_client.delete_policy(PolicyArn=policy_arn)
-        except Exception:
-            LOG.debug(f"Unable to delete policy {policy_arn=} in cleanup")
+            LOG.debug(f"Unable to delete function {arn=} in cleanup")
 
     for log_group_name in log_groups:
         try:
@@ -1139,7 +1366,7 @@ def create_lambda_function(lambda_client, logs_client, iam_client, wait_until_la
 
 @pytest.fixture
 def check_lambda_logs(logs_client):
-    def _check_logs(func_name: str, expected_lines: List[str] = None):
+    def _check_logs(func_name: str, expected_lines: List[str] = None) -> List[str]:
         if not expected_lines:
             expected_lines = []
         log_events = get_lambda_logs(func_name, logs_client=logs_client)
@@ -1150,6 +1377,7 @@ def check_lambda_logs(logs_client):
                 if any(found):
                     continue
             assert line in log_messages
+        return log_messages
 
     return _check_logs
 
@@ -1415,6 +1643,57 @@ def create_iam_role_with_policy(iam_client):
         iam_client.delete_role(RoleName=role_name)
 
 
+@pytest.fixture
+def firehose_create_delivery_stream(firehose_client, wait_for_delivery_stream_ready):
+    delivery_streams = {}
+
+    def _create_delivery_stream(**kwargs):
+        if "DeliveryStreamName" not in kwargs:
+            kwargs["DeliveryStreamName"] = f"test-delivery-stream-{short_uid()}"
+
+        delivery_stream = firehose_client.create_delivery_stream(**kwargs)
+        delivery_streams.update({kwargs["DeliveryStreamName"]: delivery_stream})
+        wait_for_delivery_stream_ready(kwargs["DeliveryStreamName"])
+        return delivery_stream
+
+    yield _create_delivery_stream
+
+    for delivery_stream_name in delivery_streams.keys():
+        firehose_client.delete_delivery_stream(DeliveryStreamName=delivery_stream_name)
+
+
+@pytest.fixture
+def events_create_rule(events_client):
+    rules = []
+
+    def _create_rule(**kwargs):
+        rule_name = kwargs["Name"]
+        bus_name = kwargs.get("EventBusName", "")
+        pattern = kwargs.get("EventPattern", {})
+        schedule = kwargs.get("ScheduleExpression", "")
+        rule_arn = events_client.put_rule(
+            Name=rule_name,
+            EventBusName=bus_name,
+            EventPattern=json.dumps(pattern),
+            ScheduleExpression=schedule,
+        )["RuleArn"]
+        rules.append({"name": rule_name, "bus": bus_name})
+        return rule_arn
+
+    yield _create_rule
+
+    for rule in rules:
+        targets = events_client.list_targets_by_rule(Rule=rule["name"], EventBusName=rule["bus"])[
+            "Targets"
+        ]
+
+        targetIds = [target["Id"] for target in targets]
+        if len(targetIds) > 0:
+            events_client.remove_targets(Rule=rule["name"], EventBusName=rule["bus"], Ids=targetIds)
+
+        events_client.delete_rule(Name=rule["name"], EventBusName=rule["bus"])
+
+
 @pytest.fixture
 def cleanups(ec2_client):
     cleanup_fns = []
@@ -1428,6 +1707,12 @@ def cleanups(ec2_client):
             LOG.warning("Failed to execute cleanup", exc_info=e)
 
 
+@pytest.fixture(scope="session")
+def account_id():
+    sts_client = _client("sts")
+    return sts_client.get_caller_identity()["Account"]
+
+
 @pytest.hookimpl
 def pytest_configure(config: Config):
     # TODO: migrate towards "whitebox" or similar structure
