@@ -9,6 +9,7 @@ import boto3
 import moto.cloudformation.utils
 import yaml
 from requests.structures import CaseInsensitiveDict
+from samtranslator.translator.managed_policy_translator import ManagedPolicyLoader
 from samtranslator.translator.transform import transform as transform_sam
 
 from localstack import config, constants
@@ -28,28 +29,30 @@ NoDatesSafeLoader.yaml_implicit_resolvers = {
     for k, v in NoDatesSafeLoader.yaml_implicit_resolvers.items()
 }
 
+policy_loader = None
+
+
+def _create_loader():
+    global policy_loader
+    if not policy_loader:
+        iam_client = aws_stack.connect_to_service("iam")
+        policy_loader = ManagedPolicyLoader(iam_client=iam_client)
+    return policy_loader
+
 
 def transform_template(req_data) -> Optional[str]:
     """only returns string when parsing SAM template, otherwise None"""
     template_body = get_template_body(req_data)
     parsed = parse_template(template_body)
     if parsed.get("Transform") == "AWS::Serverless-2016-10-31":
-        policy_map = {
-            # SAM Transformer expects this map to be non-empty, but apparently the content doesn't matter (?)
-            "dummy": "entry"
-            # 'AWSLambdaBasicExecutionRole': 'arn:aws:iam::aws:policy/service-role/AWSLambdaBasicExecutionRole',
-        }
-
-        class MockPolicyLoader:
-            def load(self):
-                return policy_map
 
         # Note: we need to fix boto3 region, otherwise AWS SAM transformer fails
         region_before = os.environ.get("AWS_DEFAULT_REGION")
         if boto3.session.Session().region_name is None:
             os.environ["AWS_DEFAULT_REGION"] = aws_stack.get_region()
+        loader = _create_loader()
         try:
-            transformed = transform_sam(parsed, {}, MockPolicyLoader())
+            transformed = transform_sam(parsed, {}, loader)
             return json.dumps(transformed)
         finally:
             os.environ.pop("AWS_DEFAULT_REGION", None)
@@ -57,7 +60,7 @@ def transform_template(req_data) -> Optional[str]:
                 os.environ["AWS_DEFAULT_REGION"] = region_before
 
 
-def prepare_template_body(req_data) -> Optional[str]:  # TODO: mutating and returning
+def prepare_template_body(req_data) -> str | bytes | None:  # TODO: mutating and returning
     template_url = req_data.get("TemplateURL")
     if template_url:
         req_data["TemplateURL"] = convert_s3_to_local_url(template_url)
@@ -136,7 +139,7 @@ def get_template_body(req_data):
             raise Exception(
                 "Unable to fetch template body (code %s) from URL %s" % (status_code, url)
             )
-        return response.content
+        return to_str(response.content)
     raise Exception("Unable to get template body from input: %s" % req_data)
 
 
