@@ -9,6 +9,8 @@ import sys
 from collections.abc import Mapping
 from typing import Any, Callable, Dict, Iterator, List, Optional, Sized, Tuple, Type, TypeVar, Union
 
+import cachetools
+
 if sys.version_info >= (3, 8):
     from typing import TypedDict
 else:
@@ -136,6 +138,17 @@ class PaginatedList(List[_ListType]):
         return result_list[start_idx : start_idx + page_size], next_token
 
 
+class CustomExpiryTTLCache(cachetools.TTLCache):
+    """TTLCache that allows to set custom expiry times for individual keys."""
+
+    def set_expiry(self, key: Any, ttl: Union[float, int]) -> float:
+        """Set the expiry of the given key in a TTLCache to (<current_time> + <ttl>)"""
+        with self.timer as time:
+            # note: need to access the internal dunder API here
+            self._TTLCache__getlink(key).expires = expiry = time + ttl
+            return expiry
+
+
 def get_safe(dictionary, path, default_value=None):
     """
     Performs a safe navigation on a Dictionary object and
@@ -401,6 +414,17 @@ def is_none_or_empty(obj: Union[Optional[str], Optional[list]]) -> bool:
     )
 
 
-def select_from_typed_dict(typed_dict: Type[TypedDict], obj: Dict):
-    """Select a subset of attributes from a dictionary based on the keys of a given `TypedDict`"""
-    return select_attributes(obj, [*typed_dict.__required_keys__, *typed_dict.__optional_keys__])
+def select_from_typed_dict(typed_dict: Type[TypedDict], obj: Dict, filter: bool = False) -> Dict:
+    """
+    Select a subset of attributes from a dictionary based on the keys of a given `TypedDict`.
+    :param typed_dict: the `TypedDict` blueprint
+    :param obj: the object to filter
+    :param filter: if True, remove all keys with an empty (e.g., empty string or dictionary) or `None` value
+    :return: the resulting dictionary (it returns a copy)
+    """
+    selection = select_attributes(
+        obj, [*typed_dict.__required_keys__, *typed_dict.__optional_keys__]
+    )
+    if filter:
+        return {k: v for k, v in selection.items() if v}
+    return selection
